/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
* EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
* SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS
* SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2018 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name   : r_mtr_ics.c
* Description : Processes of a user interface (tool)
***********************************************************************************************************************/
/**********************************************************************************************************************
* History : DD.MM.YYYY Version
*         : 28.09.2018 1.00
***********************************************************************************************************************/

/***********************************************************************************************************************
* Includes <System Includes> , "Project Includes"
***********************************************************************************************************************/
#include <machine.h>
#include <stdint.h>
#include "r_mtr_ics.h"
#include "r_mtr_config.h"
#include "r_mtr_driver_access.h"
#include "r_mtr_opl_damp_ctrl.h"
#include "r_mtr_opl2less.h"

#if     defined(RX23T_MRSSK)
    #include "ICS_RX23T.h"
#elif   defined(RX24T_MRSSK)
    #include "ICS_RX24T.h"
#elif   defined(RX24U_MRSSK)
    #include "ICS_RX24U.h"
#elif   defined(RX66T_MRSSK)
    #include "ICS2_RX66T.h"
#else
    #error Please select a valid CPU board by defining a <MCU>_<MRSSK> macro in r_mtr_config.h
#endif

static uint8_t u1_cnt_ics;                         /* Counter for period of calling "scope_watchpoint" */
static mtr_ctrl_input_t  st_ctrl_input;            /* Structure for ICS input */

/***********************************************************************************************************************
* Exported global variables (to be accessed by other files)
***********************************************************************************************************************/
/* User command variables */
uint8_t      com_u1_direction;                     /* Rotational direction (0:CW ,1:CCW) */
int16_t      com_s2_ref_speed_rpm;                 /* Motor speed reference [rpm] (mechanical) */
uint16_t     com_u2_max_speed_rpm;                 /* Maximum speed [rpm] (mechanical) */
uint16_t     com_u2_overspeed_limit_rpm;           /* Over speed limit [rpm] (mechanical) */
uint16_t     com_u2_offset_calc_time;              /* Current offset calculation time */
uint16_t     com_u2_mtr_pp;                        /* Pole pairs */
float        com_f4_mtr_r;                         /* Resistance [ohm] */
float        com_f4_mtr_ld;                        /* D-axis inductance [H] */
float        com_f4_mtr_lq;                        /* Q-axis inductance [H] */
float        com_f4_mtr_m;                         /* Permanent magnetic flux [Wb] */
float        com_f4_mtr_j;                         /* Rotor inertia [kgm^2] */
float        com_f4_current_omega;                 /* Natural frequency for current loop [Hz] */
float        com_f4_current_zeta;                  /* Damping ratio for current loop */
float        com_f4_speed_omega;                   /* Natural frequency for speed loop [Hz] */
float        com_f4_speed_zeta;                    /* Damping ratio for speed loop */
float        com_f4_e_obs_omega;                   /* Natural frequency for BEMF observer [Hz] */
float        com_f4_e_obs_zeta;                    /* Damping ratio for BEMF observer */
float        com_f4_pll_est_omega;                 /* Natural frequency for rotor position Phase-Locked Loop [Hz] */
float        com_f4_pll_est_zeta;                  /* Damping ratio for rotor position Phase-Locked Loop */
float        com_f4_id_kp;
float        com_f4_id_ki;
float        com_f4_iq_kp;
float        com_f4_iq_ki;
float        com_f4_speed_kp;
float        com_f4_speed_ki;
float        com_f4_ref_id;                        /* Id reference when open loop [A] */
float        com_f4_id_up_time;                    /* Time to increase id */
float        com_f4_limit_speed_change;            /* Limit of speed change */
float        com_f4_nominal_current_rms;           /* The nominal current [Arms] */
uint16_t     com_u2_id_up_speed_rpm;
uint16_t     com_u2_id_down_speed_rpm;
float        com_f4_id_down_time;
float        com_f4_switch_phase_err_deg;
float        com_f4_opl2less_sw_time;
float        com_f4_ed_hpf_omega;
float        com_f4_ol_damping_zeta;
float        com_f4_ol_damping_fb_limit_rate;
float        com_f4_phase_err_lpf_cut_freq;
uint8_t      com_u1_less_switch;
uint8_t      com_u1_flux_weakening;
uint8_t      com_u1_volt_err_comp;
uint8_t      com_u1_openloop_damping;
uint8_t      com_u1_enable_write;                  /* ICS write enable flag */
uint8_t      g_u1_enable_write;                    /* ICS write enable flag */

/***********************************************************************************************************************
* Function Name : mtr_set_com_variables
* Description   : Set com variables to "tool" input buffer
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void mtr_set_com_variables(void)
{
    /***** limit variable *****/
clrpsw_i();                                             /* Interrupt disable */
    com_s2_ref_speed_rpm = R_MTR_LimitAbs(com_s2_ref_speed_rpm, com_u2_max_speed_rpm);
setpsw_i();                                             /* Interrupt enable */

    /***** When com_s2_enable_sw and g_u1_enable_write are same value, rewrite enable. *****/
    if (com_u1_enable_write == g_u1_enable_write)
    {
        /* Reference speed [rpm] (mechanical) */
        st_ctrl_input.s2_ref_speed_rpm = com_s2_ref_speed_rpm;

        /* Rotation direction */
        if (R_MTR_GetDir(MTR_ID_A) != com_u1_direction)
        {
            if (MTR_CCW < com_u1_direction)
            {
                com_u1_direction = MTR_CCW;
            }
        }
        st_ctrl_input.u1_direction = com_u1_direction;

        st_ctrl_input.u1_less_switch = com_u1_less_switch;
        st_ctrl_input.u1_flux_weakening = com_u1_flux_weakening;
        st_ctrl_input.u1_volt_err_comp = com_u1_volt_err_comp;
        st_ctrl_input.u1_openloop_damping = com_u1_openloop_damping;

        /* Reference maximum speed [rpm] (mechanical) */
        st_ctrl_input.u2_max_speed_rpm = com_u2_max_speed_rpm;

        /* Over speed limit [rpm] (mechanical) */
        st_ctrl_input.u2_overspeed_limit_rpm = com_u2_overspeed_limit_rpm;

        /* Current offset calculation time */
        st_ctrl_input.u2_offset_calc_time = com_u2_offset_calc_time;

        /* Motor parameters */
        st_ctrl_input.st_motor.u2_mtr_pp = com_u2_mtr_pp;
        st_ctrl_input.st_motor.f4_mtr_r  = com_f4_mtr_r;
        st_ctrl_input.st_motor.f4_mtr_ld = com_f4_mtr_ld;
        st_ctrl_input.st_motor.f4_mtr_lq = com_f4_mtr_lq;
        st_ctrl_input.st_motor.f4_mtr_m  = com_f4_mtr_m;
        st_ctrl_input.st_motor.f4_mtr_j  = com_f4_mtr_j;

        /* Id reference when low speed */
        st_ctrl_input.f4_ref_id = com_f4_ref_id;

        /* Time to increase id */
        st_ctrl_input.f4_id_up_time = com_f4_id_up_time;

        /* Limit of speed change */
        st_ctrl_input.f4_limit_speed_change = com_f4_limit_speed_change;

        /* Natural frequency & damping ratio for control loop */
        st_ctrl_input.st_design_params.f4_current_omega = com_f4_current_omega;
        st_ctrl_input.st_design_params.f4_current_zeta  = com_f4_current_zeta;
        st_ctrl_input.st_design_params.f4_speed_omega   = com_f4_speed_omega;
        st_ctrl_input.st_design_params.f4_speed_zeta    = com_f4_speed_zeta;

        /* Natural frequency & damping ratio for BEMF observer */
        st_ctrl_input.st_design_params.f4_e_obs_omega = com_f4_e_obs_omega;
        st_ctrl_input.st_design_params.f4_e_obs_zeta = com_f4_e_obs_zeta;

        /* Natural frequency & damping ratio for PLL speed estimate loop */
        st_ctrl_input.st_design_params.f4_pll_est_omega = com_f4_pll_est_omega;
        st_ctrl_input.st_design_params.f4_pll_est_zeta = com_f4_pll_est_zeta;

        /* Control gain */
        st_ctrl_input.st_ctrl_gain.f4_id_kp    = com_f4_id_kp;
        st_ctrl_input.st_ctrl_gain.f4_id_ki    = com_f4_id_ki;
        st_ctrl_input.st_ctrl_gain.f4_iq_kp    = com_f4_iq_kp;
        st_ctrl_input.st_ctrl_gain.f4_iq_ki    = com_f4_iq_ki;
        st_ctrl_input.st_ctrl_gain.f4_speed_kp = com_f4_speed_kp;
        st_ctrl_input.st_ctrl_gain.f4_speed_ki = com_f4_speed_ki;

        /* Nominal current [Arms]*/
        st_ctrl_input.f4_nominal_current_rms = com_f4_nominal_current_rms;

        /* Speed to start increasing id */
        st_ctrl_input.u2_id_up_speed_rpm = com_u2_id_up_speed_rpm;

        /* Speed to start decreasing id */
        st_ctrl_input.u2_id_down_speed_rpm = com_u2_id_down_speed_rpm;

        /* Time to decrease id */
        st_ctrl_input.f4_id_down_time = com_f4_id_down_time;

        /* Phase error to decide sensor-less switch timing */
        st_ctrl_input.f4_switch_phase_err_deg = com_f4_switch_phase_err_deg;

        /* Time to switch open-loop to sensor-less */
        st_ctrl_input.f4_opl2less_sw_time = com_f4_opl2less_sw_time;

        /* Cut off frequency of ed HPF */
        st_ctrl_input.st_design_params.f4_ed_hpf_omega = com_f4_ed_hpf_omega;

        /* Damping ratio of open-loop damping control */
        st_ctrl_input.st_design_params.f4_ol_damping_zeta = com_f4_ol_damping_zeta;

        /* Rate of reference speed for open-loop damping control feedback speed limiter */
        st_ctrl_input.f4_ol_damping_fb_limit_rate = com_f4_ol_damping_fb_limit_rate;

        /* Cut off frequency of phase error LPF [Hz] */
        st_ctrl_input.st_design_params.f4_phase_err_lpf_cut_freq = com_f4_phase_err_lpf_cut_freq;

        /* Copy variables com to ICS input */
        R_MTR_CtrlInput(&st_ctrl_input);

        g_u1_enable_write ^= 1;                         /* Change every time 0 and 1 */
    }
} /* End of function mtr_set_com_variables */

/***********************************************************************************************************************
* Function Name : mtr_ics_variables_init
* Description   : Initialize valiables for Analyzer interface
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void mtr_ics_variables_init(void)
{
    u1_cnt_ics                      = 0;
    com_u1_direction                = MTR_CW;
    com_s2_ref_speed_rpm            = 0;
    com_u2_max_speed_rpm            = MTR_MAX_SPEED_RPM;
    com_u2_overspeed_limit_rpm      = MTR_OVERSPEED_LIMIT_RPM;
    com_u2_offset_calc_time         = MTR_OFFSET_CALC_TIME;
    com_u2_mtr_pp                   = MTR_POLE_PAIRS;
    com_f4_mtr_r                    = MTR_R;
    com_f4_mtr_ld                   = MTR_LD;
    com_f4_mtr_lq                   = MTR_LQ;
    com_f4_mtr_m                    = MTR_M;
    com_f4_mtr_j                    = MTR_J;
    com_f4_nominal_current_rms      = MTR_NOMINAL_CURRENT_RMS;
    com_f4_ref_id                   = MTR_OL_ID_REF;
    com_f4_id_up_time               = MTR_ID_UP_TIME;
    com_f4_limit_speed_change       = MTR_LIMIT_SPEED_CHANGE;
    com_f4_current_omega            = MTR_CURRENT_OMEGA;
    com_f4_current_zeta             = MTR_CURRENT_ZETA;
    com_f4_speed_omega              = MTR_SPEED_OMEGA;
    com_f4_speed_zeta               = MTR_SPEED_ZETA;
    com_f4_e_obs_omega              = MTR_E_OBS_OMEGA;
    com_f4_e_obs_zeta               = MTR_E_OBS_ZETA;
    com_f4_pll_est_omega            = MTR_PLL_EST_OMEGA;
    com_f4_pll_est_zeta             = MTR_PLL_EST_ZETA;
    com_f4_id_kp                    = 0.0f;
    com_f4_id_ki                    = 0.0f;
    com_f4_iq_kp                    = 0.0f;
    com_f4_iq_ki                    = 0.0f;
    com_f4_speed_kp                 = 0.0f;
    com_f4_speed_ki                 = 0.0f;
    com_u2_id_up_speed_rpm          = MTR_ID_UP_SPEED_RPM;
    com_u2_id_down_speed_rpm        = MTR_ID_DOWN_SPEED_RPM;
    com_f4_id_down_time             = MTR_ID_DOWN_TIME;
    com_f4_switch_phase_err_deg     = MTR_SWITCH_PHASE_ERR_DEG;
    com_f4_opl2less_sw_time         = MTR_OPL2LESS_SW_TIME;
    com_f4_ed_hpf_omega             = MTR_ED_HPF_OMEGA;
    com_f4_ol_damping_zeta          = MTR_OL_DAMPING_ZETA;
    com_f4_ol_damping_fb_limit_rate = MTR_FB_SPEED_LIMIT_RATE;
    com_f4_phase_err_lpf_cut_freq   = MTR_PHASE_ERR_LPF_CUT_FREQ;
    com_u1_less_switch              = DEFAULT_LESS_SWITCH;
    com_u1_flux_weakening           = DEFAULT_FLUX_WEAKENING;
    com_u1_volt_err_comp            = DEFAULT_VOLT_ERR_COMP;
    com_u1_openloop_damping         = DEFAULT_OPENLOOP_DAMPING;
    com_u1_enable_write             = 0;
    g_u1_enable_write               = 0;

    st_ctrl_input.u1_direction                               = com_u1_direction;
    st_ctrl_input.u1_less_switch                             = com_u1_less_switch;
    st_ctrl_input.u1_flux_weakening                          = com_u1_flux_weakening;
    st_ctrl_input.u1_volt_err_comp                           = com_u1_volt_err_comp;
    st_ctrl_input.u1_openloop_damping                        = com_u1_openloop_damping;
    st_ctrl_input.s2_ref_speed_rpm                           = com_s2_ref_speed_rpm;
    st_ctrl_input.u2_max_speed_rpm                           = com_u2_max_speed_rpm;
    st_ctrl_input.u2_overspeed_limit_rpm                     = com_u2_overspeed_limit_rpm;
    st_ctrl_input.u2_offset_calc_time                        = com_u2_offset_calc_time;
    st_ctrl_input.st_motor.u2_mtr_pp                         = com_u2_mtr_pp;
    st_ctrl_input.st_motor.f4_mtr_r                          = com_f4_mtr_r;
    st_ctrl_input.st_motor.f4_mtr_ld                         = com_f4_mtr_ld;
    st_ctrl_input.st_motor.f4_mtr_lq                         = com_f4_mtr_lq;
    st_ctrl_input.st_motor.f4_mtr_m                          = com_f4_mtr_m;
    st_ctrl_input.st_motor.f4_mtr_j                          = com_f4_mtr_j;
    st_ctrl_input.f4_ref_id                                  = com_f4_ref_id;
    st_ctrl_input.f4_id_up_time                              = com_f4_id_up_time;
    st_ctrl_input.f4_limit_speed_change                      = com_f4_limit_speed_change;
    st_ctrl_input.st_design_params.f4_current_omega          = com_f4_current_omega;
    st_ctrl_input.st_design_params.f4_current_zeta           = com_f4_current_zeta;
    st_ctrl_input.st_design_params.f4_speed_omega            = com_f4_speed_omega;
    st_ctrl_input.st_design_params.f4_speed_zeta             = com_f4_speed_zeta;
    st_ctrl_input.st_design_params.f4_e_obs_omega            = com_f4_e_obs_omega;
    st_ctrl_input.st_design_params.f4_e_obs_zeta             = com_f4_e_obs_zeta;
    st_ctrl_input.st_design_params.f4_pll_est_omega          = com_f4_pll_est_omega;
    st_ctrl_input.st_design_params.f4_pll_est_zeta           = com_f4_pll_est_zeta;
    st_ctrl_input.st_ctrl_gain.f4_id_kp                      = com_f4_id_kp;
    st_ctrl_input.st_ctrl_gain.f4_id_ki                      = com_f4_id_ki;
    st_ctrl_input.st_ctrl_gain.f4_iq_kp                      = com_f4_iq_kp;
    st_ctrl_input.st_ctrl_gain.f4_iq_ki                      = com_f4_iq_ki;
    st_ctrl_input.st_ctrl_gain.f4_speed_kp                   = com_f4_speed_kp;
    st_ctrl_input.st_ctrl_gain.f4_speed_ki                   = com_f4_speed_ki;
    st_ctrl_input.f4_nominal_current_rms                     = com_f4_nominal_current_rms;
    st_ctrl_input.u2_id_up_speed_rpm                         = com_u2_id_up_speed_rpm;
    st_ctrl_input.u2_id_down_speed_rpm                       = com_u2_id_down_speed_rpm;
    st_ctrl_input.f4_id_down_time                            = com_f4_id_down_time;
    st_ctrl_input.f4_switch_phase_err_deg                    = com_f4_switch_phase_err_deg;
    st_ctrl_input.f4_opl2less_sw_time                        = com_f4_opl2less_sw_time;
    st_ctrl_input.st_design_params.f4_ed_hpf_omega           = com_f4_ed_hpf_omega;
    st_ctrl_input.st_design_params.f4_ol_damping_zeta        = com_f4_ol_damping_zeta;
    st_ctrl_input.f4_ol_damping_fb_limit_rate                = com_f4_ol_damping_fb_limit_rate;
    st_ctrl_input.st_design_params.f4_phase_err_lpf_cut_freq = com_f4_phase_err_lpf_cut_freq;
} /* End of function mtr_ics_variables_init */

/***********************************************************************************************************************
* Function Name : mtr_ics_interrupt_process
* Description   : Process for ICS(Analyzer)
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void mtr_ics_interrupt_process(void)
{
    u1_cnt_ics++;
    if (MTR_ICS_DECIMATION <= u1_cnt_ics)                      /* Decimation of ICS call */
    {
        u1_cnt_ics = 0;
        ics2_watchpoint();                                     /* Call ICS */
    }

    /* Update commands and configurations when trigger flag is set */
    R_MTR_UpdatePolling();
} /* End of function mtr_ics_interrupt_process */
