/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
* EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
* SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS
* SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2018 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name   : r_mtr_ctrl_rx66t.c
* Description : The processes of MCU & inverter control layer for RX66T
***********************************************************************************************************************/
/***********************************************************************************************************************
* History : DD.MM.YYYY Version
*         : 28.09.2018 1.00
***********************************************************************************************************************/

/***********************************************************************************************************************
* Includes <System Includes> , "Project Includes"
***********************************************************************************************************************/
#include <stdint.h>
#include <machine.h>
#include "r_mtr_ctrl_rx66t.h"
#include "r_mtr_ctrl_mcu.h"
#include "iodefine.h"
#include "r_mtr_common.h"
#include "r_mtr_parameter.h"

/***********************************************************************************************************************
* Private functions definitions
***********************************************************************************************************************/
static void    mtr_unused_pin_setup(void);     /* Setup unused pins */
static void    mtr_init_clock(void);           /* Initializes clock */
static void    mtr_init_wdt(void);             /* Initializes WDT */
static void    mtr_init_cmt(void);             /* Initializes CMT */
static void    mtr_init_poe3(void);            /* Initializes POE3 */
#if MTR_GPT
static void    mtr_init_gpt_4_5_6(void);       /* Initialize GPTW_4_5_6 */
#else
static void    mtr_init_mtu_3_4(void);         /* Initialize MTU3_3 MTU3_4 */
#endif /* MTR_GPT */
static void    mtr_init_ad_converter(void);         /* Initialize A/D converter */
static void    mtr_init_flash_memory(void);         /* Initialize Flash Memory */
static void    mtr_init_register(uint8_t u1_id);    /* Removes unintentional first pulse */
static void    mtr_disable_int_source_select(void); /* Interrupt Source Select is disabled */

/***********************************************************************************************************************
* Function Name : R_MTR_InitHardware
* Description   : Initializes peripheral function
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_InitHardware(void)
{
    /*=========================*/
    /*     Unused pins setup   */
    /*=========================*/
    mtr_unused_pin_setup();

    /*=========================*/
    /*     Initialize clock    */
    /*=========================*/
    mtr_init_clock();

    /*========================*/
    /*     Initialize WDT     */
    /*========================*/
    mtr_init_wdt();

    /*========================*/
    /*     Initialize CMT     */
    /*========================*/
    mtr_init_cmt();

    /*=========================*/
    /*     Initialize POE3     */
    /*=========================*/
    mtr_init_poe3();

#if MTR_GPT
    /*========================================*/
    /*     Initialize GPTW_4,GPTW_5,GPTW_6    */
    /*========================================*/
    mtr_init_gpt_4_5_6();
#else
    /*===================================*/
    /*     Initialize MTU3_3,MTU3_4      */
    /*===================================*/
    mtr_init_mtu_3_4();
#endif /* MTR_GPT */

    /*==================================*/
    /*     Initialize A/D converter     */
    /*==================================*/
    mtr_init_ad_converter();

    /*=================================*/
    /*     Disable Int Source Select   */
    /*=================================*/
    mtr_disable_int_source_select();

    /*==================================*/
    /*     Initialize Flash Memory      */
    /*==================================*/
    mtr_init_flash_memory();
} /* End of function R_MTR_InitHardware */

/***********************************************************************************************************************
* Function Name : R_MTR_InitBoardUi
* Description   : Initializes peripheral function for board user interface
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_InitBoardUi(void)
{
    /*************** SW1,2 ***************/
    PORT8.PDR.BIT.B0 = 0;                  /* Input mode for SW1 */
    PORT8.PDR.BIT.B1 = 0;                  /* Input mode for SW2 */

    /*************** LED1,2,3 ***************/
    PORTE.PODR.BIT.B3 = 1;                 /* LED1 = "H" */
    PORTB.PODR.BIT.B7 = 1;                 /* LED2 = "H" */
    PORTB.PODR.BIT.B3 = 1;                 /* LED3 = "H" */
    PORTE.PDR.BIT.B3 = 1;                  /* Output mode for LED1 */
    PORTB.PDR.BIT.B7 = 1;                  /* Output mode for LED2 */
    PORTB.PDR.BIT.B3 = 1;                  /* Output mode for LED3 */
} /* End of function R_MTR_InitBoardUi */

/***********************************************************************************************************************
* Function Name : R_MTR_ClearWdt
* Description   : Clears watch dog timer
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_ClearWdt(void)
{
    IWDT.IWDTRR = 0x00;                    /* IWDT Refresh Operation */
    IWDT.IWDTRR = 0xFF;
} /* End of function R_MTR_ClearWdt */

/***********************************************************************************************************************
* Function Name : mtr_init_clock
* Description   : Initializes clock
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
static void mtr_init_clock(void)
{
    /* ---- Enable write protection ---- */
    /* PRCR - Protect Register
     b15:b8  PRKEY - PRC Key Code - A5h
     (The write value should be A5h to permission writing PRCi bit)
     b7:b4   Reserved - The write value should be 0.
     b3      PRC3 - Protect Bit 3 - Write disabled
     b2      Reserved - The write value should be 0.
     b1      PRC1 - Protect Bit 1 - Write enabled
     b0      PRC0 - Protect Bit 0 - Write enabled */
    SYSTEM.PRCR.WORD = 0xA503;

    /* ---- Turn off the HOCO power supply ---- */
    /* HOCOPCR - High-Speed On-Chip Oscillator Power Supply Control Register
     b7:b1   Reserved - The write value should be 0.
     b0      HOCOPCNT - High-Speed On-Chip Oscillator Power Supply Control
                      - HOCO power supply is turned off. */
    SYSTEM.HOCOPCR.BYTE = 0x01;

    /* ----  Set the main clock oscillator drive capability ---- */
    /* MOFCR - Main Clock Oscillator Forced Oscillation Control Register
     b7       Reserved - The write value should be 0.
     b6       MOSEL    - Main Clock Oscillator Switch - Resonator
     b5:b4    MODRV2   - Main Clock Oscillator Drive Capability 2 Switching - 8MHz
     b3:b0    Reserved - The write value should be 0. */
    SYSTEM.MOFCR.BYTE = 0x30;

    while (0x30 != SYSTEM.MOFCR.BYTE)
    {
        /* Confirm that the written value can be read correctly. */
    }

    /* ---- Set wait time until the main clock oscillator stabilizes ---- */
    /* MOSCWTCR - Main Clock Oscillator Wait Control Register
     b7:b0    MSTS     - Main Clock Oscillator Waiting Time
                       - Wait time is 2635 cycles (approx. 9.98 ms). */
    SYSTEM.MOSCWTCR.BYTE = 0x53;

    /* ---- Operate the main clock oscillator ---- */
    /* MOSCCR   - Main Clock Oscillator Control Register
     b7:b1    Reserved - The write value should be 0.
     b0       MOSTP    - Main Clock Oscillator Stop - Main clock oscillator is operating. */
    SYSTEM.MOSCCR.BYTE = 0x00;

    while (0x00 != SYSTEM.MOSCCR.BYTE)
    {
        /* Confirm that the written value can be read correctly. */
    }

    /* ---- Wait processing for the clock oscillation stabilization ---- */
    /* OSCOVFSR - Oscillation Stabilization Flag Register
     b7:b5  Reserved - The write value should be 0.
     b4     ILCOVF - IWDT Low-Speed Clock Oscillation Stabilization Flag
     b3     HCOVF - HOCO Clock Oscillation Stabilization Flag
     b2     PLOVF - PLL Clock Oscillation Stabilization Flag
     b1     Reserved - The write value should be 0.
     b0     MOOVF - Main Clock Oscillation Stabilization Flag */
    while (1 != SYSTEM.OSCOVFSR.BIT.MOOVF)
    {
        /* Confirm that the Oscillation of the main clock is stable so the clock is
         available for use as the system clock.*/
    }

    /* ---- Memory wait cycle ---- */
    /* MEMWAIT - Memory Wait Cycle Setting Register
     b7:b1  Reserved - These bits are read as 0. The write value should be 0.
     b0     MEMWAIT  - Memory Wait Cycle Setting - 0: No wait
                                                 - 1: One wait cycle */
    SYSTEM.MEMWAIT.BYTE = 0x01;

    while (0x01 != SYSTEM.MEMWAIT.BYTE)
    {
        /* Wait for writing MEMWAIT */
    }

    /* ---- Set the PLL division ratio and multiplication factor ---- */
    /* PLLCR - PLL Control Register
     b15:b14  Reserved  - The write value should be 0.
     b13:b8   STC       - Frequency Multiplication Factor Select
                        - Frequency multiplication factor is multiply-by-20.
     b7:b5    Reserved  - The write value should be 0.
     b4       PLLSRCSEL - PLL Clock Source Select
                        - Main clock oscillator
     b1:b0    PLIDIV    - PLL Input Frequency Division Ratio Select
     - PLL input division ratio is no division. */
    SYSTEM.PLLCR.WORD = 0x2700;

    /* ---- Operate the PLL clock oscillator ---- */
    /* PLLCR2 - PLL Control Register 2
     b7:b1    Reserved - The write value should be 0.
     b0       PLLEN    - PLL Stop Control - PLL is operating. */
    SYSTEM.PLLCR2.BYTE = 0x00;

    /* ---- Wait processing for the clock oscillation stabilization ---- */
    while (1 != SYSTEM.OSCOVFSR.BIT.PLOVF)
    {
        /* Confirm that the Oscillation of the pll clock is stable so the clock is
         available for use as the system clock.*/
    }

    /* ---- Set the internal clock division ratio ---- */
    /* SCKCR - System Clock Control Register
     b31:b28 FCK      - FlashIF Clock(FCLK) Select - divide-by-4
     b27:b24 ICK      - System Clock (ICLK) Select - no division
     b23     PSTOP1   - BCLK Pin Output Control    - disabled. (Fixed high)
     b22:b20 Reserved - The write value should be 0.
     b19:b16 BCK      - External Bus Clock (BCLK) Select - divide-by-4
     b15:b12 PCLKA    - Peripheral Module Clock A(PCLKA) Select - divide-by-2
     b11:b8  PCLKB    - Peripheral Module Clock B(PCLKB) Select - divide-by-4
     b7:b4   PCLKC    - Peripheral Module Clock C(PCLKC) Select - no division
     b3:b0   PCLKD    - Peripheral Module Clock D(PCLKD) Select - divide-by-4 */
    SYSTEM.SCKCR.LONG = 0x20821202;

    while (0x20821202 != SYSTEM.SCKCR.LONG)
    {
        /* Confirm that the written value can be read correctly. */
    }

    /* ---- Set the BCLK pin output ---- */
    /* BCKCR - External Bus Clock Control Register
     b7:b1   Reserved - The write value should be 0.
     b0      BCLKDIV  - BCLK Pin Output Select - no division */
    SYSTEM.BCKCR.BYTE = 0x00;

    while (0x00 != SYSTEM.BCKCR.BYTE)
    {
        /* Confirm that the written value can be read correctly. */
    }

    /* ---- Set the internal clock division ratio(USB) ---- */
    /* SCKCR2 - System Clock Control Register 2
     b15:b8 Reserved - The write value should be 0.
     b7:b4  UCK      - USB Clock Select - When USB is not in use, these bits are read as 0001b.
                     - The write value should be 0001b.
     b3:b1  Reserved - The write value should be 0.
     b0     Reserved - The write value should be 1. */
    SYSTEM.SCKCR2.WORD = 0x0011;

    while (0x0011 != SYSTEM.SCKCR2.WORD)
    {
        /* Confirm that the written value can be read correctly. */
    }

    /* ---- Set the internal clock source ---- */
    /* SCKCR3 - System Clock Control Register 3
     b15:b11 Reserved - The write value should be 0.
     b10:b8  CKSEL    - Clock Source Select - PLL circuit is selected.
     b7:b1   Reserved - The write value should be 0. */
    SYSTEM.SCKCR3.WORD = 0x0400;

    while (0x0400 != SYSTEM.SCKCR3.WORD)
    {
        /* Confirm that the written value can be read correctly. */
    }

    /* ---- Voltage level setting register ---- */
    /* VOLSR - Voltage Level Setting Register 
     b7  RICVLS - RIIC Voltage Level Setting - Specifies the VCC voltage level to use the RIIC module.
                                             - 0: 4.5V <= VCC <= 5.5V
                                             - 1: 2.7V <= VCC <  4.5V
     b6  PGAVLS - PGA Voltage Level Setting  - 0: To use differential input of PGA AVCC is between 4.5 V and 5.5 V.
                                             - 1: To set other than those above
     b5:b3 Reserved - The write value should be 0.
     b2  USBVON - USB Power Supply Turning-On Setting - 0: When USB not in use (Deep software standby mode included)
                                                      - 1: When USB in use
     b1:b0 Reserved - The write value should be 0. */

    SYSTEM.VOLSR.BYTE = 0x40;

    while (0x40 != SYSTEM.VOLSR.BYTE)
    {
        /* Confirm that the written value can be read correctly. */
    }

    /* ---- Disable write protection ---- */
    /* PRCR - Protect Register
     b15:b8 PRKEY - PRC Key Code - A5h
     (The write value should be A5h to permission writing PRCi bit)
     b2     Reserved - The write value should be 0.
     b1     PRC1 - Protect Bit 1 - Write disabled
     b0     PRC0 - Protect Bit 0 - Write disabled */
    SYSTEM.PRCR.WORD = 0xA500;
} /* End of function mtr_init_clock */

/***********************************************************************************************************************
* Function Name : mtr_init_wdt
* Description   : Initializes IWDT
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
static void mtr_init_wdt(void)
{
    SYSTEM.PRCR.WORD = 0xA501;             /* PRCR write enabled */
    SYSTEM.ILOCOCR.BIT.ILCSTP = 0;         /* IWDT-dedicated on-chip oscillator is operating */
    SYSTEM.PRCR.WORD = 0xA500;             /* PRCR write disabled */
    IWDT.IWDTRR = 0x00;                    /* IWDT Refresh Operation */
    IWDT.IWDTRR = 0xFF;
} /* End of function mtr_init_wdt */

/***********************************************************************************************************************
* Function Name : mtr_init_cmt
* Description   : Initializes CMT
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
static void mtr_init_cmt(void)
{
    SYSTEM.PRCR.WORD = 0xA502;                      /* Enable write protection */
    MSTP_CMT0 = 0;                                  /* The module stop state is canceled */
    SYSTEM.PRCR.WORD = 0xA500;                      /* Disable write protection */

    CMT.CMSTR0.BIT.STR0 = 0;                        /* CMT0 count is stopped */
    CMT0.CMCR.BIT.CKS = 0;                          /* Clock : PCLK/8 = 5[MHz] */
    CMT0.CMCOR = 2499;                              /* Set interval period : 500 [us] */
                                                    /* 2499 = 5[MHz] * 500[us] - 1 */

    ICU.IPR[IPR_CMT0_CMI0].BIT.IPR = 11;            /* Interrupt priority level 11 */
    ICU.IR[IR_CMT0_CMI0].BIT.IR = 0;                /* Clear interrupt request */
    ICU.IER[IER_CMT0_CMI0].BIT.IEN_CMT0_CMI0 = 1;   /* Interrupt request enable */
    CMT0.CMCR.BIT.CMIE = 1;                         /* Compare match interrupt (CMI0) enabled */

    CMT.CMSTR0.BIT.STR0 = 1;                        /* CMT0 count is started */
} /* End of function mtr_init_cmt */

/***********************************************************************************************************************
* Function Name : mtr_init_poe3
* Description   : Initializes POE3 (Over-current protection)
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
static void mtr_init_poe3(void)
{
    PORT7.PMR.BIT.B0 = 0;                  /* Use pin as general I/O port */

    MPC.PWPR.BIT.B0WI = 0;                 /* Writing to the PFSWE bit is enabled */
    MPC.PWPR.BIT.PFSWE = 1;                /* Writing to the PFS register is enabled */

    MPC.P70PFS.BIT.PSEL = 7;               /* Set peripheral for POE0# */

    MPC.PWPR.BIT.PFSWE = 0;                /* Writing to the PFS register is disabled */
    MPC.PWPR.BIT.B0WI = 1;                 /* Writing to the PFSWE bit is disabled */

    PORT7.PMR.BIT.B0 = 1;                  /* Use pin as I/O port for peripheral functions */

    POE.ICSR1.BIT.POE0M = 0;               /* Accepts a request on the falling edge of POE0# input */
    POE.ICSR1.BIT.PIE1 = 1;                /* Interrupt requests by the input level detection enabled */

#if MTR_GPT
    POE.ALR4.WORD = 0x00BF;                /* Set GPT_4-6 output pins for initial output"L",active level"H" */
#else
    POE.ALR1.WORD = 0x00BF;                /* Set MTU_3-4 output pins for initial output"L",active level"H" */
#endif /* MTR_GPT */

    ICU.IPR[IPR_ICU_GROUPBL1].BIT.IPR = 15;                 /* Interrupt priority level 15 */
    ICU.IR[IR_ICU_GROUPBL1].BIT.IR = 0;                     /* Clear interrupt request */
    ICU.IER[IER_ICU_GROUPBL1].BIT.IEN_ICU_GROUPBL1 = 1;     /* Interrupt request enable */
    ICU.GENBL1.BIT.EN_POE3_OEI1 = 1;
#if MTR_GPT
    POE.OCSR4.BIT.OCE4 = 1;                /* Simultaneous Conduction Output Disable */
    POE.OCSR4.BIT.OIE4 = 1;                /* Simultaneous Conduction Interrupt Enable */
#else
    POE.OCSR1.BIT.OCE1 = 1;                /* Simultaneous Conduction Output Disable */
    POE.OCSR1.BIT.OIE1 = 1;                /* Simultaneous Conduction Interrupt Enable */
#endif /* MTR_GPT */
} /* End of function mtr_init_poe3 */

#if MTR_GPT
/******************************************************************************
* Function Name : mtr_init_gpt_4_5_6
* Description   : Initialize GPTW PWM output
* Arguments     : none
* Return Value  : none
******************************************************************************/
void mtr_init_gpt_4_5_6(void)
{
    /***** module stop state *****/
    SYSTEM.PRCR.WORD = 0xA502;                          /* enable write protection */
    MSTP_GPTW = 0;                                      /* the module stop state is canceled */
    SYSTEM.PRCR.WORD = 0xA500;                          /* disable write protection */

    /***** Stop count operation *****/
    GPTW4.GTSTP.LONG = 0xFFFFFFFF;                      /* stop GPT Timer Counter */

    /***** PWM mode selection *****/
    GPTW4.GTCR.BIT.MD = 0x5;                            /* GPT4 : PWM Mode2 */
    GPTW5.GTCR.BIT.MD = 0x5;                            /* GPT5 : PWM Mode2 */
    GPTW6.GTCR.BIT.MD = 0x5;                            /* GPT6 : PWM Mode2 */

    /***** Count clock selection *****/
    GPTW4.GTCR.BIT.TPCS = 0;                            /* GPT4 clock : PCLKC */
    GPTW5.GTCR.BIT.TPCS = 0;                            /* GPT5 clock : PCLKC */
    GPTW6.GTCR.BIT.TPCS = 0;                            /* GPT6 clock : PCLKC */

    /***** GTPR setting *****/
    GPTW4.GTPR  = MTR_CARRIER_SET_BASE;                 /* GPT4 : set carrier period */
    GPTW5.GTPR  = MTR_CARRIER_SET_BASE;                 /* GPT5 : set carrier period */
    GPTW6.GTPR  = MTR_CARRIER_SET_BASE;                 /* GPT6 : set carrier period */
    GPTW4.GTPBR = MTR_CARRIER_SET_BASE;                 /* GPT4 : set carrier period */
    GPTW5.GTPBR = MTR_CARRIER_SET_BASE;                 /* GPT5 : set carrier period */
    GPTW6.GTPBR = MTR_CARRIER_SET_BASE;                 /* GPT6 : set carrier period */

    /***** GTCNT setting *****/
    GPTW4.GTCNT = 0;                                    /* set "0" */
    GPTW5.GTCNT = 0;                                    /* set "0" */
    GPTW6.GTCNT = 0;                                    /* set "0" */

    /*****GTIOC pin output enable setting *****/
    GPTW4.GTIOR.BIT.OAE = 0;                            /* Output disable */
    GPTW4.GTIOR.BIT.OBE = 0;                            /* Output disable */
    GPTW5.GTIOR.BIT.OAE = 0;                            /* Output disable */
    GPTW5.GTIOR.BIT.OBE = 0;                            /* Output disable */
    GPTW6.GTIOR.BIT.OAE = 0;                            /* Output disable */
    GPTW6.GTIOR.BIT.OBE = 0;                            /* Output disable */

    /***** GTIOC terminal function setting *****/
    GPTW4.GTIOR.BIT.GTIOA = 0x03;                       /* 00011b */
    GPTW4.GTIOR.BIT.GTIOB = 0x13;                       /* 10011b */
    GPTW5.GTIOR.BIT.GTIOA = 0x03;                       /* 00011b */
    GPTW5.GTIOR.BIT.GTIOB = 0x13;                       /* 10011b */ 
    GPTW6.GTIOR.BIT.GTIOA = 0x03;                       /* 00011b */ 
    GPTW6.GTIOR.BIT.GTIOB = 0x13;                       /* 10011b */ 

    /***** MPC setting *****/
    PORT7.PMR.BIT.B1 = 0;                               /* use pin as general I/O port */
    PORT7.PMR.BIT.B2 = 0;                               /* use pin as general I/O port */
    PORT7.PMR.BIT.B3 = 0;                               /* use pin as general I/O port */
    PORT7.PMR.BIT.B4 = 0;                               /* use pin as general I/O port */
    PORT7.PMR.BIT.B5 = 0;                               /* use pin as general I/O port */
    PORT7.PMR.BIT.B6 = 0;                               /* use pin as general I/O port */

    PORT7.PODR.BIT.B1 = 0;                              /* Up = "L" */
    PORT7.PODR.BIT.B2 = 0;                              /* Vp = "L" */
    PORT7.PODR.BIT.B3 = 0;                              /* Wp = "L" */
    PORT7.PODR.BIT.B4 = 0;                              /* Un = "L" */
    PORT7.PODR.BIT.B5 = 0;                              /* Vn = "L" */
    PORT7.PODR.BIT.B6 = 0;                              /* Wn = "L" */
    PORT7.PDR.BIT.B1 = 1;                               /* output mode for Up */
    PORT7.PDR.BIT.B2 = 1;                               /* output mode for Vp */
    PORT7.PDR.BIT.B3 = 1;                               /* output mode for Wp */
    PORT7.PDR.BIT.B4 = 1;                               /* output mode for Un */
    PORT7.PDR.BIT.B5 = 1;                               /* output mode for Vn */
    PORT7.PDR.BIT.B6 = 1;                               /* output mode for Wn */

    MPC.PWPR.BIT.B0WI = 0;                              /* Writing to the PFSWE bit is enabled */
    MPC.PWPR.BIT.PFSWE = 1;                             /* Writing to the PFS register is enabled */

    MPC.P71PFS.BIT.PSEL = 0x15;                         /* set peripheral for GTIOC4A */
    MPC.P72PFS.BIT.PSEL = 0x15;                         /* set peripheral for GTIOC5A */
    MPC.P73PFS.BIT.PSEL = 0x15;                         /* set peripheral for GTIOC6A */
    MPC.P74PFS.BIT.PSEL = 0x15;                         /* set peripheral for GTIOC4B */
    MPC.P75PFS.BIT.PSEL = 0x15;                         /* set peripheral for GTIOC5B */
    MPC.P76PFS.BIT.PSEL = 0x15;                         /* set peripheral for GTIOC6B */

    MPC.PWPR.BIT.PFSWE = 0;                             /* Writing to the PFS register is disabled */
    MPC.PWPR.BIT.B0WI = 1;                              /* Writing to the PFSWE bit is disabled */

    PORT7.PMR.BIT.B1 = 1;                               /* use pin as I/O port for peripheral functions */
    PORT7.PMR.BIT.B2 = 1;                               /* use pin as I/O port for peripheral functions */
    PORT7.PMR.BIT.B3 = 1;                               /* use pin as I/O port for peripheral functions */
    PORT7.PMR.BIT.B4 = 1;                               /* use pin as I/O port for peripheral functions */
    PORT7.PMR.BIT.B5 = 1;                               /* use pin as I/O port for peripheral functions */
    PORT7.PMR.BIT.B6 = 1;                               /* use pin as I/O port for peripheral functions */

    /* GTCCRn buffer operation setting*/
    GPTW4.GTBER.BIT.CCRA = 1;                           /* single buffer */
    GPTW4.GTBER.BIT.CCRB = 1;                           /* single buffer */
    GPTW5.GTBER.BIT.CCRA = 1;                           /* single buffer */
    GPTW5.GTBER.BIT.CCRB = 1;                           /* single buffer */
    GPTW6.GTBER.BIT.CCRA = 1;                           /* single buffer */
    GPTW6.GTBER.BIT.CCRB = 1;                           /* single buffer */

    /***** GTCCRn setting (Set initial duty)*****/
    GPTW4.GTCCRA = MTR_CARRIER_SET_BASE;
    GPTW4.GTCCRB = MTR_CARRIER_SET_BASE;
    GPTW5.GTCCRA = MTR_CARRIER_SET_BASE;
    GPTW5.GTCCRB = MTR_CARRIER_SET_BASE;
    GPTW6.GTCCRA = MTR_CARRIER_SET_BASE;
    GPTW6.GTCCRB = MTR_CARRIER_SET_BASE;

    GPTW4.GTCCRC = MTR_CARRIER_SET_BASE;
    GPTW4.GTCCRE = MTR_CARRIER_SET_BASE;
    GPTW5.GTCCRC = MTR_CARRIER_SET_BASE;
    GPTW5.GTCCRE = MTR_CARRIER_SET_BASE;
    GPTW6.GTCCRC = MTR_CARRIER_SET_BASE;
    GPTW6.GTCCRE = MTR_CARRIER_SET_BASE;

    /***** Deadtime function selection *****/
    GPTW4.GTDTCR.BIT.TDE = 1;                           /* Use GTDVU and GTDVD */
    GPTW5.GTDTCR.BIT.TDE = 1;                           /* Use GTDVU and GTDVD */
    GPTW6.GTDTCR.BIT.TDE = 1;                           /* Use GTDVU and GTDVD */
    /***** Set deadtime value *****/
    GPTW4.GTDVU = MTR_DEADTIME_SET;                     /* set deadtime */
    GPTW4.GTDVD = MTR_DEADTIME_SET;                     /* set deadtime */
    GPTW5.GTDVU = MTR_DEADTIME_SET;                     /* set deadtime */
    GPTW5.GTDVD = MTR_DEADTIME_SET;                     /* set deadtime */
    GPTW6.GTDVU = MTR_DEADTIME_SET;                     /* set deadtime */
    GPTW6.GTDVD = MTR_DEADTIME_SET;                     /* set deadtime */

    ICU.SLIAR208.BYTE = 0x64;                           /* Set the interrupt source number (0x64:GTCIU4) */

    ICU.IPR[IPR_PERIA_INTA208].BIT.IPR = 12;                /* Interrupt priority level 12 */
    ICU.IR[IR_PERIA_INTA208].BIT.IR = 0;                    /* Clear interrupt request */
    ICU.IER[IER_PERIA_INTA208].BIT.IEN_PERIA_INTA208 = 1;   /* Interrupt request is enable */

    GPTW4.GTINTAD.BIT.GTINTPR = 2;                          /* Enable interrupt request in valleyy */
    GPTW4.GTINTAD.BIT.ADTRADEN = 1;                         /* Enable A/D start request with compare match (down count) to GTADTRA */
    GPTW4.GTADTRA = 0x00000000;                             /* A/D conversion start request timing setting */

    GPTW4.GTITC.BIT.IVTC = 2;                               /* Skipping interrupts with count of valleys */
    GPTW4.GTITC.BIT.IVTT = MTR_INT_DECIMATION;              /* Interrupt skip number of time */
    GPTW4.GTITC.BIT.ADTAL = 1;                              /* enable A/D convert request */

    /***** Start count operation *****/
    GPTW4.GTSSR.BIT.CSTRT = 1;
    GPTW5.GTSSR.BIT.CSTRT = 1;
    GPTW6.GTSSR.BIT.CSTRT = 1;
    GPTW4.GTSTR.LONG = 0x00000070;                          /* start GPT_4-6 */
}/* End of function mtr_init_gpt_4_5_6 */
#else /* MTR_GPT */

/***********************************************************************************************************************
* Function Name : mtr_init_mtu_3_4
* Description   : Initializes MTU3 timer for PWM output
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
static void mtr_init_mtu_3_4(void)
{
    /***** Module stop state *****/
    SYSTEM.PRCR.WORD = 0xA502;             /* Enable write protection */
    MSTP_MTU = 0;                          /* The module stop state is canceled */
    SYSTEM.PRCR.WORD = 0xA500;             /* Disable write protection */

    /***** Stop count operation *****/
    MTU.TSTRA.BYTE = 0x00;                 /* Stop MTU3_3 and MTU3_4 */

    /***** Count clock selection *****/
    MTU3.TCR.BIT.TPSC   = 0;               /* MTU3_3 clock : PCLKA */
    MTU3.TCR.BIT.CKEG   = 0;               /* MTU3_3 count at rising edge */
    MTU3.TCR2.BIT.TPSC2 = 0;
    MTU4.TCR.BIT.TPSC   = 0;               /* MTU3_4 clock : PCLKA */
    MTU4.TCR.BIT.CKEG   = 0;               /* MTU3_4 count at rising edge */
    MTU4.TCR2.BIT.TPSC2 = 0;

    /***** TCNT setting *****/
    MTU3.TCNT = MTR_DEADTIME_SET;          /* Set deadtime */
    MTU4.TCNT = 0;                         /* Set "0" */

    /***** TGR setting *****/
    MTU3.TGRB = (uint16_t)(MTR_CARRIER_SET_BASE + (MTR_DEADTIME_SET * 0.5f));
    MTU4.TGRA = (uint16_t)(MTR_CARRIER_SET_BASE + (MTR_DEADTIME_SET * 0.5f));
    MTU4.TGRB = (uint16_t)(MTR_CARRIER_SET_BASE + (MTR_DEADTIME_SET * 0.5f));
                                           /* Set initial duty */
    MTU3.TGRD = (uint16_t)(MTR_CARRIER_SET_BASE + (MTR_DEADTIME_SET * 0.5f));
    MTU4.TGRC = (uint16_t)(MTR_CARRIER_SET_BASE + (MTR_DEADTIME_SET * 0.5f));
    MTU4.TGRD = (uint16_t)(MTR_CARRIER_SET_BASE + (MTR_DEADTIME_SET * 0.5f));
                                           /* Set initial duty */

    /***** Dead-time and carrier period setting *****/
    MTU.TDDRA = MTR_DEADTIME_SET;          /* Set deadtime */

    MTU.TCDRA = (uint16_t)(MTR_CARRIER_SET - MTR_DEADTIME_SET);
    MTU.TCBRA = (uint16_t)(MTR_CARRIER_SET - MTR_DEADTIME_SET);
                                           /* Set carrier period */

    MTU3.TGRA = MTR_CARRIER_SET;           /* Set "carrier period + deadtime" */
    MTU3.TGRC = MTR_CARRIER_SET;

    /***** Enable PWM output, set PWM output level *****/
    MTU.TOCR1A.BIT.PSYE = 0;               /* Toggle output is disabled */
    MTU.TOCR1A.BIT.OLSP = 1;               /* Positive phase : initial output"L",active level"H" */
    MTU.TOCR1A.BIT.OLSN = 1;               /* Negative phase : initial output"L",active level"H" */

    /***** Complementary PWM mode setting *****/
    MTU3.TMDR1.BIT.MD = 13;                /* MD[3:0]=0b1101 : complementary PWM mode (transfer at crest) */
    MTU3.TMDR1.BIT.BFA = 1;                /* TGRA and TGRC used together for buffer operation */
    MTU3.TMDR1.BIT.BFB = 1;                /* TGRB and TGRD used together for buffer operation */

    /***** Waveform output setting *****/
    MTU.TOERA.BYTE = 0xC0;                 /* PWM output disable  */
    MTU3.TIORH.BYTE = 0x00;                /* MTIOC3B output prohibited */
    MTU3.TIORL.BYTE = 0x00;                /* MTIOC3D output prohibited */
    MTU4.TIORH.BYTE = 0x00;                /* MTIOC4B output prohibited */
    MTU4.TIORL.BYTE = 0x00;                /* MTIOC4D output prohibited */

    /***** MPC setting *****/
    PORT7.PMR.BIT.B1 = 0;                  /* Use pin as general I/O port */
    PORT7.PMR.BIT.B2 = 0;                  /* Use pin as general I/O port */
    PORT7.PMR.BIT.B3 = 0;                  /* Use pin as general I/O port */
    PORT7.PMR.BIT.B4 = 0;                  /* Use pin as general I/O port */
    PORT7.PMR.BIT.B5 = 0;                  /* Use pin as general I/O port */
    PORT7.PMR.BIT.B6 = 0;                  /* Use pin as general I/O port */

    PORT7.PODR.BIT.B1 = 0;                 /* Up = "L" */
    PORT7.PODR.BIT.B2 = 0;                 /* Vp = "L" */
    PORT7.PODR.BIT.B3 = 0;                 /* Wp = "L" */
    PORT7.PODR.BIT.B4 = 0;                 /* Un = "L" */
    PORT7.PODR.BIT.B5 = 0;                 /* Vn = "L" */
    PORT7.PODR.BIT.B6 = 0;                 /* Wn = "L" */
    PORT7.PDR.BIT.B1 = 1;                  /* Output mode for Up */
    PORT7.PDR.BIT.B2 = 1;                  /* Output mode for Vp */
    PORT7.PDR.BIT.B3 = 1;                  /* Output mode for Wp */
    PORT7.PDR.BIT.B4 = 1;                  /* Output mode for Un */
    PORT7.PDR.BIT.B5 = 1;                  /* Output mode for Vn */
    PORT7.PDR.BIT.B6 = 1;                  /* Output mode for Wn */

    MPC.PWPR.BIT.B0WI = 0;                 /* Writing to the PFSWE bit is enabled */
    MPC.PWPR.BIT.PFSWE = 1;                /* Writing to the PFS register is enabled */

    MPC.P71PFS.BIT.PSEL = 1;               /* Set peripheral for MTIOC3B */
    MPC.P72PFS.BIT.PSEL = 1;               /* Set peripheral for MTIOC4A */
    MPC.P73PFS.BIT.PSEL = 1;               /* Set peripheral for MTIOC4B */
    MPC.P74PFS.BIT.PSEL = 1;               /* Set peripheral for MTIOC3D */
    MPC.P75PFS.BIT.PSEL = 1;               /* Set peripheral for MTIOC4C */
    MPC.P76PFS.BIT.PSEL = 1;               /* Set peripheral for MTIOC4D */

    MPC.PWPR.BIT.PFSWE = 0;                /* Writing to the PFS register is disabled */
    MPC.PWPR.BIT.B0WI = 1;                 /* Writing to the PFSWE bit is disabled */

    PORT7.PMR.BIT.B1 = 1;                  /* Use pin as I/O port for peripheral functions */
    PORT7.PMR.BIT.B2 = 1;                  /* Use pin as I/O port for peripheral functions */
    PORT7.PMR.BIT.B3 = 1;                  /* Use pin as I/O port for peripheral functions */
    PORT7.PMR.BIT.B4 = 1;                  /* Use pin as I/O port for peripheral functions */
    PORT7.PMR.BIT.B5 = 1;                  /* Use pin as I/O port for peripheral functions */
    PORT7.PMR.BIT.B6 = 1;                  /* Use pin as I/O port for peripheral functions */

    /***** Interrupt setting *****/
    MTU4.TIER.BIT.TCIEV = 1;               /* Enable underflow interrupt when operation is in complementary PWM mode */
    MTU4.TIER.BIT.TTGE2 = 1;               /* Enable TCNT underflow A/D convert */
    MTU4.TIER.BIT.TTGE = 0;                /* Disable A/D convert request */

    ICU.SLIAR208.BYTE = 0x19;                               /* Set the interrupt source number (0x19:TCIV4) */
    ICU.IPR[IPR_PERIA_INTA208].BIT.IPR = 12;                /* Interrupt priority level 12 */
    ICU.IR[IR_PERIA_INTA208].BIT.IR = 0;                    /* Clear interrupt request */
    ICU.IER[IER_PERIA_INTA208].BIT.IEN_PERIA_INTA208 = 1;   /* Interrupt request is enable */

    MTU.TITMRA.BIT.TITM = 0;                                /* Set interrupt skipping function */
    MTU.TITCR1A.BIT.T4VCOR = MTR_INT_DECIMATION;            /* Interrupt skip number of time */
    MTU.TITCR1A.BIT.T4VEN = 1;                              /* Enable TCIV4 interrupt skip */

    /***** Start count operation *****/
    MTU.TSTRA.BYTE = 0xC0;                 /* Start MTU3_3 and MTU3_4 */
} /* End of function mtr_init_mtu_3_4 */
#endif /* MTR_GPT */

/***********************************************************************************************************************
* Function Name : mtr_init_ad_converter
* Description   : Initializes A/D converter
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
static void mtr_init_ad_converter(void)
{
    SYSTEM.PRCR.WORD = 0xA502;             /* Enable write protection */
    MSTP_S12AD = 0;                        /* The module stop state is canceled */
    MSTP_S12AD2 = 0;                       /* The module stop state is canceled */
#if MTR_GPT
    MSTP_ELC = 0;                          /* The module stop state is canceled */
#endif /* MTR_GPT */
    SYSTEM.PRCR.WORD = 0xA500;             /* Disable write protection */

    S12AD.ADCSR.BIT.EXTRG = 0;             /* Use single-cycle scan mode with sample-and-hold function */
    S12AD.ADCSR.BIT.TRGE = 1;              /* Enables A/D conversion trigger */
    S12AD.ADCSR.BIT.ADIE = 0;              /* Disables S12ADI interrupt generation upon scan completion */
    S12AD.ADCSR.BIT.ADCS = 0;              /* Single scan mode */

    S12AD2.ADCSR.BIT.EXTRG = 0;            /* Use single-cycle scan mode with sample-and-hold function */
    S12AD2.ADCSR.BIT.TRGE = 1;             /* Disables A/D conversion trigger */
    S12AD2.ADCSR.BIT.ADIE = 0;             /* Disables S12ADI2 interrupt generation upon scan completion */
    S12AD2.ADCSR.BIT.ADCS = 0;             /* Single scan mode */

    /* ---- PGA differential input disabled or enabled  ---- */
    S12AD.ADPGADCR0.WORD = 0x0000;
    while (0x0000 != S12AD.ADPGADCR0.WORD)
    {
        /* Confirm that the written value can be read correctly. */
    }

    MPC.PWPR.BIT.B0WI = 0;                 /* Writing to the PFSWE bit is enabled */
    MPC.PWPR.BIT.PFSWE = 1;                /* Writing to the PFS register is enabled */

    MPC.P40PFS.BIT.ASEL = 1;               /* Pin Function Select : AN000 (IU) */
    MPC.P42PFS.BIT.ASEL = 1;               /* Pin Function Select : AN002 (IW) */
    MPC.P62PFS.BIT.ASEL = 1;               /* Pin Function Select : AN208 (VPN) */
    MPC.P21PFS.BIT.ASEL = 1;               /* Pin Function Select : AN217 (VR1) */

    MPC.PWPR.BIT.PFSWE = 0;                /* Writing to the PFS register is disabled */
    MPC.PWPR.BIT.B0WI = 1;                 /* Writing to the PFSWE bit is disabled */

    S12AD.ADANSA0.WORD = 0x0005;           /* A/D convert channel select AN000,AN002 */
    S12AD2.ADANSA0.WORD = 0x0100;          /* A/D convert channel select AN208 */
    S12AD2.ADANSA1.WORD = 0x0002;          /* A/D convert channel select AN217 */

    S12AD.ADSHCR.BIT.SHANS = 5;            /* Enable sampling & hold AN000 and AN002 */
    S12AD.ADCER.BIT.ADRFMT = 0;            /* A/D data format "aligning to the right" */
    S12AD2.ADCER.BIT.ADRFMT = 0;           /* A/D data format "aligning to the right" */

    S12AD.ADPGACR.WORD = 0x0101;           /* Input to the A/D converter : AN000,AN002 */

#if MTR_GPT
    ELC.ELCR.BIT.ELCON = 1;                /* ELC function is enabled */
    ELC.ELSR15.BIT.ELS = 0x78;             /* GPTW4 A/D converter start request A */
    ELC.ELSR46.BIT.ELS = 0x78;             /* GPTW4 A/D converter start request A */
    S12AD.ADSTRGR.BIT.TRSA = 0x32;         /* A/D convert trigger A/D Startup source 0 from ELC */
    S12AD2.ADSTRGR.BIT.TRSA = 0x32;        /* A/D convert trigger A/D Startup source 0 from ELC */
#else
    S12AD.ADSTRGR.BIT.TRSA = 5;            /* A/D convert trigger MTU4.TCNT underflow */
    S12AD2.ADSTRGR.BIT.TRSA = 5;           /* A/D convert trigger MTU4.TCNT underflow */
#endif /* MTR_GPT */
} /* End of function mtr_init_ad_converter */

/***********************************************************************************************************************
* Function Name : mtr_init_register
* Description   : Avoid unintentional first pulse
* Arguments     : u1_id - Motor ID
* Return Value  : None
***********************************************************************************************************************/
static void mtr_init_register(uint8_t u1_id)
{
    mtr_inv_set_uvw(0.5f, 0.5f, 0.5f, u1_id);
} /* End of function mtr_init_register */

/***********************************************************************************************************************
* Function Name : mtr_init_flash_memory
* Description   : Enables ROM Cache
* Arguments     : none
* Return Value  : none
***********************************************************************************************************************/
static void mtr_init_flash_memory(void)
{
    /******************************/
    /*  ROM Cache Enable Process  */
    /******************************/
    FLASH.ROMCIV.BIT.ROMCIV = 1;                        /* Start ROM Cache invalidation. */
    while (1 == FLASH.ROMCIV.BIT.ROMCIV)
    {
        /* Wait for the ROMCIV.ROMCIV bit to become 0. (1: Invalidation is in progress.) */
    }
    FLASH.ROMCE.BIT.ROMCEN = 1;                         /* ROM cache operation enabled. */
    while (0 == FLASH.ROMCE.BIT.ROMCEN)
    {
        /* Confirm that the ROMCE.ROMCEN bit is 1. */
    }
} /* End of function mtr_init_flash_memory */

/***********************************************************************************************************************
* Function Name : mtr_unused_pin_setup
* Description   : Setups unused pins
*                 If an unused pin is floating, it's direction should be set to "output"
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
static void mtr_unused_pin_setup(void)
{
    /* Set Unused pins to output mode */
    PORT0.PDR.BIT.B0 = 1;                               /* Pin No.4 */
    PORT0.PDR.BIT.B1 = 1;                               /* Pin No.7 */
    PORT2.PDR.BIT.B0 = 1;                               /* Pin No.69 */
    PORT2.PDR.BIT.B2 = 1;                               /* Pin No.67 */
    PORT2.PDR.BIT.B3 = 1;                               /* Pin No.66 */
    PORT2.PDR.BIT.B4 = 1;                               /* Pin No.65 */
    PORT2.PDR.BIT.B7 = 1;                               /* Pin No.64 */
    PORT3.PDR.BIT.B0 = 1;                               /* Pin No.63 */
    PORT3.PDR.BIT.B1 = 1;                               /* Pin No.61 */
    PORT8.PDR.BIT.B2 = 1;                               /* Pin No.96 */
    PORT9.PDR.BIT.B0 = 1;                               /* Pin No.50 */
    PORT9.PDR.BIT.B1 = 1;                               /* Pin No.49 */
    PORT9.PDR.BIT.B2 = 1;                               /* Pin No.48 */
    PORT9.PDR.BIT.B3 = 1;                               /* Pin No.47 */
    PORT9.PDR.BIT.B4 = 1;                               /* Pin No.46 */
    PORT9.PDR.BIT.B5 = 1;                               /* Pin No.45 */
    PORTA.PDR.BIT.B0 = 1;                               /* Pin No.41 */
    PORTA.PDR.BIT.B1 = 1;                               /* Pin No.40 */
    PORTA.PDR.BIT.B2 = 1;                               /* Pin No.39 */
    PORTA.PDR.BIT.B3 = 1;                               /* Pin No.38 */
    PORTA.PDR.BIT.B4 = 1;                               /* Pin No.37 */
    PORTB.PDR.BIT.B2 = 1;                               /* Pin No.33 */
    PORTB.PDR.BIT.B6 = 1;                               /* Pin No.27 */
    PORTD.PDR.BIT.B0 = 1;                               /* Pin No.25 */
    PORTD.PDR.BIT.B1 = 1;                               /* Pin No.24 */
    PORTD.PDR.BIT.B2 = 1;                               /* Pin No.23 */
    PORTD.PDR.BIT.B3 = 1;                               /* Pin No.22 */
    PORTD.PDR.BIT.B4 = 1;                               /* Pin No.21 */
    PORTD.PDR.BIT.B5 = 1;                               /* Pin No.20 */
    PORTD.PDR.BIT.B6 = 1;                               /* Pin No.19 */
    PORTD.PDR.BIT.B7 = 1;                               /* Pin No.18 */
    PORTE.PDR.BIT.B4 = 1;                               /* Pin No.8 */
} /* End of function mtr_unused_pin_setup */

/***********************************************************************************************************************
* Function Name : mtr_ctrl_start
* Description   : Starts PWM output
* Arguments     : u1_id - Motor ID
* Return Value  : None
***********************************************************************************************************************/
void mtr_ctrl_start(uint8_t u1_id)
{
    mtr_init_register(u1_id);
    switch (u1_id)
    {
        case MTR_ID_A:
            MTR_PORT_UP = 0;                            /* Up = "L" */
            MTR_PORT_UN = 0;                            /* Un = "L" */
            MTR_PORT_VP = 0;                            /* Vp = "L" */
            MTR_PORT_VN = 0;                            /* Vn = "L" */
            MTR_PORT_WP = 0;                            /* Wp = "L" */
            MTR_PORT_WN = 0;                            /* Wn = "L" */

            PORT7.PMR.BIT.B1 = 1;                       /* I/O port for peripheral functions */
            PORT7.PMR.BIT.B2 = 1;                       /* I/O port for peripheral functions */
            PORT7.PMR.BIT.B3 = 1;                       /* I/O port for peripheral functions */
            PORT7.PMR.BIT.B4 = 1;                       /* I/O port for peripheral functions */
            PORT7.PMR.BIT.B5 = 1;                       /* I/O port for peripheral functions */
            PORT7.PMR.BIT.B6 = 1;                       /* I/O port for peripheral functions */
#if MTR_GPT
            GPTW4.GTIOR.BIT.OAE = 1;                    /* Output enable */
            GPTW4.GTIOR.BIT.OBE = 1;                    /* Output enable */
            GPTW5.GTIOR.BIT.OAE = 1;                    /* Output enable */
            GPTW5.GTIOR.BIT.OBE = 1;                    /* Output enable */
            GPTW6.GTIOR.BIT.OAE = 1;                    /* Output enable */
            GPTW6.GTIOR.BIT.OBE = 1;                    /* Output enable */
#else
            MTU.TOERA.BYTE = 0xFF;                      /* PWM output enable */
#endif /* MTR_GPT */
        break;

        default:
            /* Do Nothing */
        break;
    }
} /* End of function mtr_ctrl_start */

/***********************************************************************************************************************
* Function Name : mtr_ctrl_stop
* Description   : Stops PWM output disable for motor control
* Arguments     : u1_id - Motor ID
* Return Value  : None
***********************************************************************************************************************/
void mtr_ctrl_stop(uint8_t u1_id)
{
    switch (u1_id)
    {
        case MTR_ID_A:
#if MTR_GPT
            GPTW4.GTIOR.BIT.OAE = 0;                    /* Output disable */
            GPTW4.GTIOR.BIT.OBE = 0;                    /* Output disable */
            GPTW5.GTIOR.BIT.OAE = 0;                    /* Output disable */
            GPTW5.GTIOR.BIT.OBE = 0;                    /* Output disable */
            GPTW6.GTIOR.BIT.OAE = 0;                    /* Output disable */
            GPTW6.GTIOR.BIT.OBE = 0;                    /* Output disable */
#else
            MTU.TOERA.BYTE = 0xC0;                      /* PWM output disable */
#endif /* MTR_GPT */
            PORT7.PMR.BIT.B1 = 0;                       /* General I/O port */
            PORT7.PMR.BIT.B2 = 0;                       /* General I/O port */
            PORT7.PMR.BIT.B3 = 0;                       /* General I/O port */
            PORT7.PMR.BIT.B4 = 0;                       /* General I/O port */
            PORT7.PMR.BIT.B5 = 0;                       /* General I/O port */
            PORT7.PMR.BIT.B6 = 0;                       /* General I/O port */
            MTR_PORT_UP = 0;                            /* Up = "L" */
            MTR_PORT_UN = 0;                            /* Un = "L" */
            MTR_PORT_VP = 0;                            /* Vp = "L" */
            MTR_PORT_VN = 0;                            /* Vn = "L" */
            MTR_PORT_WP = 0;                            /* Wp = "L" */
            MTR_PORT_WN = 0;                            /* Wn = "L" */
        break;

        default:
            /* Do Nothing */
        break;
    }
    mtr_init_register(u1_id);
} /* End of function mtr_ctrl_stop */

/***********************************************************************************************************************
* Function Name : mtr_get_vr1_adc
* Description   : Gets the A/D conversion result for VR1
* Arguments     : None
* Return Value  : A/D converted value
***********************************************************************************************************************/
uint16_t mtr_get_vr1_adc(void)
{
    uint16_t u2_temp;

    while (1 == S12AD2.ADCSR.BIT.ADST)
    {
        /* Wait until AD conversion finish */
    }
    u2_temp = S12AD2.ADDR17;

    return (u2_temp);
} /* End of function mtr_get_vr1_adc */

/***********************************************************************************************************************
* Function Name : mtr_inv_set_uvw
* Description   : PWM duty setting
* Arguments     : f4_duty_u - The duty cycle of Phase-U (0.0 - 1.0)
*                 f4_duty_v - The duty cycle of Phase-V (0.0 - 1.0)
*                 f4_duty_w - The duty cycle of Phase-W (0.0 - 1.0)
*                 u1_id     - The ID of motor connector
* Return Value  : None
***********************************************************************************************************************/
void mtr_inv_set_uvw(float f4_duty_u, float f4_duty_v, float f4_duty_w, uint8_t u1_id)
{
    uint16_t u2_count_u;
    uint16_t u2_count_v;
    uint16_t u2_count_w;

    u2_count_u = (uint16_t)((MTR_CARRIER_SET_BASE * (1.0f - f4_duty_u)) + (MTR_DEADTIME_SET * 0.5f));
    u2_count_v = (uint16_t)((MTR_CARRIER_SET_BASE * (1.0f - f4_duty_v)) + (MTR_DEADTIME_SET * 0.5f));
    u2_count_w = (uint16_t)((MTR_CARRIER_SET_BASE * (1.0f - f4_duty_w)) + (MTR_DEADTIME_SET * 0.5f));

    switch (u1_id)
    {
        case MTR_ID_A:
        {
#if MTR_GPT
            /* set U-phase duty */
            GPTW4.GTCCRC = u2_count_u;
            GPTW4.GTCCRE = u2_count_u;
            /* set V-phase duty */
            GPTW5.GTCCRC = u2_count_v;
            GPTW5.GTCCRE = u2_count_v;
            /* set W-phase duty */
            GPTW6.GTCCRC = u2_count_w;
            GPTW6.GTCCRE = u2_count_w;
#else
            /* Write general registers of timer */
            MTU3.TGRD = (uint16_t)u2_count_u;
            MTU4.TGRC = (uint16_t)u2_count_v;
            MTU4.TGRD = (uint16_t)u2_count_w;
#endif /* MTR_GPT */
        break;
        }
        default:
        {
            /* Do Nothing */
        break;
        }
    }
} /* End of function mtr_inv_set_uvw */

/***********************************************************************************************************************
* Function Name : mtr_get_current_adc
* Description   : Gets the A/D conversion result for Iu,Iw
* Arguments     : f4_iu_ad - Where to store the A/D conversion result of U-phase
*                 f4_iw_ad - Where to store the A/D conversion result of W-phase
*                 u1_id - Motor ID
* Return Value  : None
***********************************************************************************************************************/
void mtr_get_current_adc(float *f4_iu_ad, float *f4_iw_ad, uint8_t u1_id)
{
    switch (u1_id)
    {
        case MTR_ID_A:
            while (1 == S12AD.ADCSR.BIT.ADST)
            {
                /* Wait until AD conversion finish */
            }
            *f4_iu_ad = (float)S12AD.ADDR0;
            *f4_iw_ad = (float)S12AD.ADDR2;
        break;

        default:
            /* Do Nothing */
        break;
    }
} /* End of function mtr_get_current_adc */

/***********************************************************************************************************************
* Function Name : mtr_get_vdc_adc
* Description   : A/D conversion for Vdc
* Arguments     : u1_id - Motor ID
* Return Value  : A/D converted value
***********************************************************************************************************************/
float mtr_get_vdc_adc(uint8_t u1_id)
{
    float f4_temp;

    switch (u1_id)
    {
        case MTR_ID_A:
            while (1 == S12AD2.ADCSR.BIT.ADST)
            {
                /* Wait until AD conversion finish */
            }
            f4_temp = (float)S12AD2.ADDR8;
        break;

        default:
            /* Do Nothing */
        break;
    }

    return (f4_temp);
} /* End of function mtr_get_vdc_adc */

/***********************************************************************************************************************
* Function Name : mtr_clear_oc_flag
* Description   : Clears forced cutoff flag (detect over current)
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void mtr_clear_oc_flag(void)
{
    ICU.IR[IR_ICU_GROUPBL1].BIT.IR = 0;             /* Release from high-impedance state */
    POE.ICSR1.BIT.POE0F = 0;                        /* POE0# high-impedance request clear */
#if MTR_GPT
    POE.OCSR4.BIT.OSF4 = 1;                         /* Output short circuit clear */
#else
    POE.OCSR1.BIT.OSF1 = 0;                         /* Output short circuit clear */
#endif /* MTR_GPT */
} /* End of function mtr_clear_oc_flag */

/***********************************************************************************************************************
* Function Name : mtr_disable_int_source_select
* Description   : Disable writing to "Software Configurable Interrupt Source Select Register"
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
static void mtr_disable_int_source_select(void)
{
    ICU.SLIPRCR.BIT.WPRC = 1;                           /* Interrupt Source Select is disabled */
    while (1 != ICU.SLIPRCR.BIT.WPRC)
    {
        /* Confirm that the written value can be read correctly. */
    }
} /* End of function mtr_disable_int_source_select */
