/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
* EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
* SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS
* SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2018 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name   : r_mtr_bemf_observer.h
* Description : Definitions of BEMF observer
***********************************************************************************************************************/
/**********************************************************************************************************************
* History : DD.MM.YYYY Version
*         : 28.09.2018 1.00
***********************************************************************************************************************/

/* Guard against multiple inclusion */
#ifndef R_MTR_BEMF_OBSERVER_H
#define R_MTR_BEMF_OBSERVER_H

#include "r_mtr_parameter.h"

/***********************************************************************************************************************
* Structure definitions
***********************************************************************************************************************/
typedef struct
{
    float f4_k_e_obs_1;                                     /* BEMF observer gain 1 */
    float f4_k_e_obs_2;                                     /* BEMF observer gain 2 */
    float f4_i_pre;                                         /* Previous value of current */
    float f4_i_est_pre;                                     /* Previous value of estimated current */
    float f4_d_est;                                         /* Estimated voltage disturbance */
    float f4_d_est_pre;                                     /* Previous value of voltage disturbance */
} mtr_bemf_obs_axis_t;

typedef struct
{
    float               f4_dt;                              /* Control period */
    mtr_parameter_t     *pmotor_params;                     /* Motor parameters */
    mtr_bemf_obs_axis_t st_d_axis;                          /* D-axis observer */
    mtr_bemf_obs_axis_t st_q_axis;                          /* Q-axis observer */
} mtr_bemf_observer_t;

typedef struct
{
    float       f4_kp_est_speed;                            /* The proportional gain for PLL */
    float       f4_ki_est_speed;                            /* The integral gain for PLL*/
    float       f4_i_est_speed;                             /* The integrator for PLL*/
} mtr_pll_est_t;

/***********************************************************************************************************************
* Global function definitions
***********************************************************************************************************************/
/***********************************************************************************************************************
* Function Name : mtr_bemf_observer
* Description   : Estimate voltage disturbance on dq-axis
* Arguments     : st_bemf_obs - BEMF observer structure (pointer)
*                 f4_vd_ref - d-axis voltage reference
*                 f4_vq_ref - q-axis voltage reference
*                 f4_id - d-axis detected current
*                 f4_iq - q-axis detected current
* Return Value  : None
***********************************************************************************************************************/
void mtr_bemf_observer(mtr_bemf_observer_t *st_bemf_obs, float f4_vd_ref, float f4_vq_ref, float f4_id, float f4_iq);

/***********************************************************************************************************************
* Function Name : mtr_bemf_calc_d
* Description   : Calculate d-axis BEMF
* Arguments     : st_bemf_obs - BEMF observer structure (pointer)
*                 f4_speed_rad - Motor speed
*                 f4_iq - q-axis detected current
* Return Value  : D-axis BEMF
***********************************************************************************************************************/
float mtr_bemf_calc_d(mtr_bemf_observer_t *st_bemf_obs, float f4_speed_rad, float f4_iq);

/***********************************************************************************************************************
* Function Name : mtr_bemf_calc_q
* Description   : Calculate q-axis BEMF
* Arguments     : st_bemf_obs - BEMF observer structure (pointer)
*                 f4_speed_rad - Motor speed
*                 f4_id - d-axis detected current
* Return Value  : Q-axis BEMF
***********************************************************************************************************************/
float mtr_bemf_calc_q(mtr_bemf_observer_t *st_bemf_obs, float f4_speed_rad, float f4_id);

/***********************************************************************************************************************
* Function Name : mtr_bemf_amp_calc
* Description   : Calculate amplitude of BEMF
* Arguments     : f4_ed - d-axis BEMF
*                 f4_eq - q-axis BEMF
* Return Value  : Amplitude of BEMF
***********************************************************************************************************************/
float mtr_bemf_amp_calc(float f4_ed, float f4_eq);

/***********************************************************************************************************************
* Function Name : mtr_bemf_obs_init
* Description   : Initialize BEMF observer
* Arguments     : st_bemf_obs - BEMF observer structure (pointer)
*                 st_motor_params - Motor parameters structure (pointer)
* Return Value  : None
***********************************************************************************************************************/
void mtr_bemf_obs_init(mtr_bemf_observer_t *st_bemf_obs, mtr_parameter_t *st_motor_params);

/***********************************************************************************************************************
* Function Name : mtr_bemf_observer_gain_calc
* Description   : Set BEMF observer gains
* Arguments     : st_motor -  Motor parameters structure(pointer)
*                 st_design_params - Design parameter structure (pointer)
*                 st_bemf_obs - BEMF observer structure (pointer)
*                 f4_ctrl_period - The control period
* Return Value  : None
***********************************************************************************************************************/
void mtr_bemf_observer_gain_calc(mtr_parameter_t *st_motor, mtr_design_parameter_t *st_design_params,
                                 mtr_bemf_observer_t *st_bemf_obs, float f4_ctrl_period);

/***********************************************************************************************************************
* Function Name : mtr_bemf_obs_set_gain
* Description   : Set BEMF observer gains
* Arguments     : st_bemf_obs - BEMF observer structure (pointer)
*                 f4_dt - Control period
*                 f4_k1_d, f4_k2_d, f4_k1_q, f4_k2_q - BEMF estimation gains
* Return Value  : None
***********************************************************************************************************************/
void mtr_bemf_obs_set_gain(mtr_bemf_observer_t *st_bemf_obs, float f4_dt,
    float f4_k1_d, float f4_k2_d, float f4_k1_q, float f4_k2_q);

/***********************************************************************************************************************
* Function Name : mtr_bemf_obs_reset
* Description   : Reset BEMF observer (except for gains and limiters)
* Arguments     : st_bemf_obs - BEMF observer structure (pointer)
* Return Value  : None
***********************************************************************************************************************/
void mtr_bemf_obs_reset(mtr_bemf_observer_t *st_bemf_obs);

/***********************************************************************************************************************
* Function Name : mtr_angle_speed_pll
* Description   : Calculate angle and speed using BEMF observer
* Arguments     : st_pll_est - PLL estimator structure (pointer)
                  f4_phase_err - Estimated phase error from actual rotor electrical angle
*                 f4_speed - Electrical speed (pointer)
* Return Value  : Estimated angle, Estimated speed
***********************************************************************************************************************/
void mtr_angle_speed_pll(mtr_pll_est_t *st_pll_est, float f4_phase_err, float *f4_speed);

/***********************************************************************************************************************
* Function Name : mtr_angle_speed_init
* Description   : Initialize angle/speed estimator
* Arguments     : st_pll_est - PLL estimator structure (pointer)
* Return Value  : None
***********************************************************************************************************************/
void mtr_angle_speed_init(mtr_pll_est_t *st_pll_est);

/***********************************************************************************************************************
* Function Name : mtr_angle_speed_gain_calc
* Description   : Set angle/speed estimator gains
* Arguments     : st_design_params - Design parameter structure (pointer)
*                 st_pll_est - PLL estimator structure (pointer)
*                 f4_ctrl_period - The control period
* Return Value  : None
***********************************************************************************************************************/
void mtr_angle_speed_gain_calc(mtr_design_parameter_t *st_design_params, mtr_pll_est_t *st_pll_est, float f4_ctrl_period);

/***********************************************************************************************************************
* Function Name : mtr_angle_speed_set_gain
* Description   : Set angle/speed estimator gains
* Arguments     : st_pll_est - PLL estimator structure (pointer)
*                 f4_kp - The proportional gain for PLL
*                 f4_ki - The integral gain for PLL
* Return Value  : None
***********************************************************************************************************************/
void mtr_angle_speed_set_gain(mtr_pll_est_t *st_pll_est, float f4_kp, float f4_ki);

/***********************************************************************************************************************
* Function Name : mtr_angle_speed_reset
* Description   : Reset angle/speed estimator (except for gains and limiters)
* Arguments     : st_pll_est - PLL estimator structure (pointer)
* Return Value  : None
***********************************************************************************************************************/
void mtr_angle_speed_reset(mtr_pll_est_t *st_pll_est);

#endif /* R_MTR_BEMF_OBSERVER_H */
