/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
* EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
* SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS
* SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2018 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name   : r_mtr_foc_control_less_speed.c
* Description : The processes of sensor-less FOC
***********************************************************************************************************************/
/**********************************************************************************************************************
* History : DD.MM.YYYY Version
*         : 28.09.2018 1.00
***********************************************************************************************************************/

/***********************************************************************************************************************
* Includes <System Includes> , "Project Includes"
***********************************************************************************************************************/
/* Standard library headers */
#include <math.h>
#include <machine.h>

/* Main associated header file */
#include "r_mtr_foc_control_less_speed.h"

/* Project headers */
#include "r_mtr_common.h"
#include "r_mtr_ctrl_mcu.h"

/***********************************************************************************************************************
* Function Name : mtr_foc_init
* Description   : Initializes FOC drive system with default configuration and specified motor ID
* Arguments     : st_foc - The pointer to the FOC data structure
*                 u1_id - The motor ID which specify the motor connectors
* Return Value  : None
***********************************************************************************************************************/
void mtr_foc_init(mtr_foc_control_t *st_foc, uint8_t u1_id)
{
    /* Initialize system state machine */
    mtr_statemachine_init(&st_foc->st_stm);

    /* Store the id to access HW resources */
    st_foc->u1_id = u1_id;

    /* Reset configurations to default */
    mtr_foc_motor_default_init(st_foc);
} /* End of function mtr_foc_init */

/***********************************************************************************************************************
* Function Name : mtr_foc_motor_default_init
* Description   : Initializes motor drive modules with default configuration
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_foc_motor_default_init(mtr_foc_control_t *st_foc)
{
    mtr_design_parameter_t st_default_parameters;

    st_default_parameters.f4_current_omega          = MTR_CURRENT_OMEGA;
    st_default_parameters.f4_current_zeta           = MTR_CURRENT_ZETA;
    st_default_parameters.f4_speed_omega            = MTR_SPEED_OMEGA;
    st_default_parameters.f4_speed_zeta             = MTR_SPEED_ZETA;
    st_default_parameters.f4_e_obs_omega            = MTR_E_OBS_OMEGA;
    st_default_parameters.f4_e_obs_zeta             = MTR_E_OBS_ZETA;
    st_default_parameters.f4_pll_est_omega          = MTR_PLL_EST_OMEGA;
    st_default_parameters.f4_pll_est_zeta           = MTR_PLL_EST_ZETA;
    st_default_parameters.f4_ed_hpf_omega           = MTR_ED_HPF_OMEGA;
    st_default_parameters.f4_ol_damping_zeta        = MTR_OL_DAMPING_ZETA;
    st_default_parameters.f4_phase_err_lpf_cut_freq = MTR_PHASE_ERR_LPF_CUT_FREQ;

    /* Initialize member of motor parameter structure */
    st_foc->st_motor.u2_mtr_pp = MTR_POLE_PAIRS;
    st_foc->st_motor.f4_mtr_r  = MTR_R;
    st_foc->st_motor.f4_mtr_ld = MTR_LD;
    st_foc->st_motor.f4_mtr_lq = MTR_LQ;
    st_foc->st_motor.f4_mtr_m  = MTR_M;
    st_foc->st_motor.f4_mtr_j  = MTR_J;

    /* Initialize offset calibration */
    st_foc->u1_flag_offset_calc      = MTR_FLG_CLR;
    st_foc->u2_offset_calc_time      = MTR_OFFSET_CALC_TIME;
    st_foc->u2_cnt_adjust            = 0;
    st_foc->f4_offset_iu             = 0.0f;
    st_foc->f4_offset_iw             = 0.0f;
    st_foc->f4_sum_iu_ad             = 0.0f;
    st_foc->f4_sum_iw_ad             = 0.0f;

    if (MTR_FLG_SET != st_foc->u1_flag_charge_cap)
    {
        st_foc->u1_flag_charge_cap = MTR_FLG_CLR;
    }
    else
    {
        /* Do Nothing */
    }
    st_foc->u1_direction             = MTR_CW;

    st_foc->f4_overspeed_limit_rad   = MTR_OVERSPEED_LIMIT_RPM;
    st_foc->f4_overvoltage_limit     = MTR_OVERVOLTAGE_LIMIT;
    st_foc->f4_undervoltage_limit    = MTR_UNDERVOLTAGE_LIMIT;
    st_foc->f4_opl2less_sw_time      = MTR_OPL2LESS_SW_TIME;
    st_foc->f4_opl_torque_current    = 0.0f;
    st_foc->u1_flag_down_to_ol       = MTR_FLG_CLR;
    st_foc->f4_switch_phase_err_rad  = MTR_SWITCH_PHASE_ERR_RAD;
    st_foc->f4_ol_id_down_step       = MTR_ID_DOWN_STEP;
    st_foc->f4_ol_id_up_step         = MTR_ID_UP_STEP;
    st_foc->f4_ol_id_ref             = MTR_OL_ID_REF;
    st_foc->f4_ol_iq_down_step_ratio = (1.0f / MTR_ID_UP_TIME);
    st_foc->f4_id_down_speed_rad     = MTR_ID_DOWN_SPEED_RAD;
    st_foc->f4_id_up_speed_rad       = MTR_ID_UP_SPEED_RAD;
    mtr_foc_phase_err_lpf_setup(st_foc, MTR_PHASE_ERR_LPF_CUT_FREQ);

    st_foc->u1_less_switch                         = DEFAULT_LESS_SWITCH;
    st_foc->u1_flux_weakening                      = DEFAULT_FLUX_WEAKENING;
    st_foc->st_volt_comp_p.u1_volt_err_comp_enable = DEFAULT_VOLT_ERR_COMP;
    st_foc->u1_openloop_damping                    = DEFAULT_OPENLOOP_DAMPING;

    /* Current control parameter reset */
    mtr_foc_current_init(&st_foc->st_cc);

    /* The dq-axis current PI gains */
    mtr_foc_current_pi_gain_calc(
        &st_foc->st_motor,
        &st_default_parameters,
        &st_foc->st_cc,
        st_foc->st_cc.f4_ctrl_period
    );

    /* Speed control parameter reset */
    mtr_speed_control_init(&st_foc->st_sc);

    /* Speed PI gains */
    mtr_speed_pi_gain_calc(&st_foc->st_motor, &st_default_parameters, &st_foc->st_sc,
            st_foc->st_sc.f4_speed_ctrl_period);

    /* Setup current limits by nominal current of motor */
    mtr_foc_set_nominal_current(st_foc, MTR_NOMINAL_CURRENT_RMS);

    /* Voltage error compensation*/
    mtr_volt_err_comp_init(&st_foc->st_volt_comp_p);

    /* Initialize member of open-loop damping control structure */
    mtr_opl_damp_init(
        &st_foc->st_opl_damp,
        MTR_FB_SPEED_LIMIT_RATE
    );

    /* Open-loop damping control gain */
    mtr_opl_damp_r_set_gain(
        &st_foc->st_opl_damp,
        st_foc->st_motor.u2_mtr_pp,
        st_foc->st_motor.f4_mtr_m,
        st_foc->st_motor.f4_mtr_j,
        st_default_parameters.f4_ol_damping_zeta,
        st_default_parameters.f4_ed_hpf_omega,
        st_foc->f4_ol_id_ref,
        st_foc->f4_id_down_speed_rad,
        st_foc->st_sc.f4_speed_ctrl_period
    );

    /* Initialize member of BEMF observer structure */
    mtr_bemf_obs_init(
        &st_foc->st_bemf_obs,
        (mtr_parameter_t*)(&st_foc->st_motor)
    );

    /* BEMF observer gain */
    mtr_bemf_observer_gain_calc(&st_foc->st_motor, &st_default_parameters, &st_foc->st_bemf_obs,
            st_foc->st_cc.f4_ctrl_period);

    /* Initialize member of angle speed structure */
    mtr_angle_speed_init(&st_foc->st_pll_est);

    /* PLL gain */
    mtr_angle_speed_gain_calc(&st_default_parameters, &st_foc->st_pll_est,
            st_foc->st_cc.f4_ctrl_period);

    /* Estimated speed LPF */
    R_MTR_FirstOrderLpffInit(&st_foc->st_est_speed_lpf);
    R_MTR_FirstOrderLpffGainCalc(&st_foc->st_est_speed_lpf,
                                 MTR_EST_SPEED_LPF_OMEGA_Hz,
                                 st_foc->st_sc.f4_speed_ctrl_period);

    /* Initialize modulation module */
    R_MOD_Init(&st_foc->st_mod);
    R_MOD_SetVdc(&st_foc->st_mod, MTR_INPUT_V);
    R_MOD_SetMaxDuty(&st_foc->st_mod, MTR_MAX_DUTY_CYCLE);
    R_MOD_SetMinDuty(&st_foc->st_mod, MTR_MIN_DUTY_CYCLE);
    R_MOD_SetVoltageErrorRatio(&st_foc->st_mod, MTR_DUTY_CYCLE_DEADTIME);

    /* Initialize Flux-weakening module */
    R_FLUXWKN_Init(&st_foc->st_fluxwkn, MTR_LIMIT_IQ, R_MOD_GetVamax(&st_foc->st_mod), &st_foc->st_motor);

    /* Reset motor */
    mtr_foc_motor_reset(st_foc);
} /* End of function mtr_foc_motor_default_init */

/***********************************************************************************************************************
* Function Name : mtr_foc_motor_reset
* Description   : Resets motor drive modules, configurations will not be reset
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_foc_motor_reset(mtr_foc_control_t *st_foc)
{
    /* Reset status structure members */
    st_foc->u2_run_mode              = MTR_MODE_INIT;
    st_foc->u2_ctrl_conf             = MTR_CONTROL_CURRENT;
    st_foc->u2_error_status          = MTR_ERROR_NONE;
    st_foc->u1_state_id_ref          = MTR_ID_UP;
    st_foc->u1_state_iq_ref          = MTR_IQ_ZERO_CONST;
    st_foc->u1_state_speed_ref       = MTR_SPEED_ZERO_CONST;

    st_foc->f4_vdc_ad                = 0.0f;
    st_foc->f4_refu                  = 0.0f;
    st_foc->f4_refv                  = 0.0f;
    st_foc->f4_refw                  = 0.0f;
    st_foc->f4_iu_ad                 = 0.0f;
    st_foc->f4_iv_ad                 = 0.0f;
    st_foc->f4_iw_ad                 = 0.0f;
    st_foc->f4_modu                  = 0.0f;
    st_foc->f4_modv                  = 0.0f;
    st_foc->f4_modw                  = 0.0f;

    st_foc->f4_ed                    = 0.0f;
    st_foc->f4_eq                    = 0.0f;
    st_foc->f4_phase_err_rad         = 0.0f;
    st_foc->f4_phase_err_rad_lpf     = 0.0f;
    st_foc->f4_init_phase_err_rad    = 0.0f;
    st_foc->f4_ol_iq_down_step       = 0.0f;

    mtr_rotor_angle_update(&st_foc->st_rotor_angle, 0.0f);
    R_MTR_FirstOrderLpffReset(&st_foc->st_phase_err_lpf);
    R_MOD_Reset(&st_foc->st_mod);

    /* Reset current control module */
    mtr_foc_current_reset(&st_foc->st_cc);

    /* Reset speed control module */
    mtr_speed_control_reset(&st_foc->st_sc);

    /* Reset BEMF observer and PLL */
    mtr_bemf_obs_reset(&st_foc->st_bemf_obs);
    mtr_angle_speed_reset(&st_foc->st_pll_est);

    /* Reset phase error LPF */
    R_MTR_FirstOrderLpffReset(&st_foc->st_phase_err_lpf);

    /* Reset estimated speed LPF */
    R_MTR_FirstOrderLpffReset(&st_foc->st_est_speed_lpf);
    st_foc->f4_speed_lpf_rad = 0.0f;

    /* Reset open-loop damping */
    st_foc->f4_damp_comp_speed = 0.0f;
    mtr_opl_damp_reset(&st_foc->st_opl_damp);

    /* Voltage error compensation */
    mtr_volt_err_comp_reset(&st_foc->st_volt_comp_p);

    st_foc->u1_enable_fluxwkn_flag = 0;
    R_FLUXWKN_Reset(&st_foc->st_fluxwkn);

    st_foc->f4_opl_torque_current = 0.0f;
} /* End of function mtr_foc_motor_reset */

/***********************************************************************************************************************
* Function Name : mtr_foc_error
* Description   : Sets error flags and trigger error event when u2_error_flag is not zero
* Arguments     : st_foc - The pointer to the FOC data structure
*                 u2_error_flag - The error flags of the errors to report
* Return Value  : None
***********************************************************************************************************************/
void mtr_foc_error(mtr_foc_control_t *st_foc, uint16_t u2_error_flag)
{
    if (MTR_ERROR_NONE != u2_error_flag)
    {
        st_foc->u2_error_status |= u2_error_flag;
        mtr_statemachine_event(&st_foc->st_stm, st_foc, MTR_EVENT_ERROR);
    }
} /* End of function mtr_foc_error */

/***********************************************************************************************************************
* Function Name : mtr_calib_current_offset
* Description   : Measures the current offset
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_calib_current_offset(mtr_foc_control_t *st_foc)
{
    float f4_neutral_duty;

    f4_neutral_duty = st_foc->st_mod.f4_neutral_duty;
    mtr_inv_set_uvw(f4_neutral_duty, f4_neutral_duty, f4_neutral_duty, st_foc->u1_id);

    /*==================================*/
    /* Current offset calibration       */
    /*==================================*/
    st_foc->u2_cnt_adjust++;
    if (st_foc->u2_offset_calc_time <= st_foc->u2_cnt_adjust)
    {
        st_foc->u1_flag_offset_calc = MTR_FLG_SET;
        st_foc->u2_cnt_adjust = st_foc->u2_offset_calc_time;
        st_foc->f4_offset_iu = st_foc->f4_sum_iu_ad / st_foc->u2_offset_calc_time;
        st_foc->f4_offset_iw = st_foc->f4_sum_iw_ad / st_foc->u2_offset_calc_time;
    }
    else
    {
        st_foc->f4_sum_iu_ad += st_foc->f4_iu_ad;
        st_foc->f4_sum_iw_ad += st_foc->f4_iw_ad;
    }
} /* End of function mtr_calib_current_offset */

/***********************************************************************************************************************
* Function Name : mtr_current_offset_adjustment
* Description   : Calibrates the current offset
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_current_offset_adjustment(mtr_foc_control_t *st_foc)
{
    /* Current offset adjustment */
    st_foc->f4_iu_ad = st_foc->f4_iu_ad - st_foc->f4_offset_iu;
    st_foc->f4_iw_ad = st_foc->f4_iw_ad - st_foc->f4_offset_iw;
    st_foc->f4_iv_ad = - (st_foc->f4_iu_ad + st_foc->f4_iw_ad);
} /* End of function mtr_current_offset_adjustment */

/***********************************************************************************************************************
* Function Name : mtr_decide_direction
* Description   : Updates direction by sign of electrical speed
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_decide_direction(mtr_foc_control_t *st_foc)
{
    if (0.0f <= st_foc->f4_speed_lpf_rad)
    {
        st_foc->u1_direction = MTR_CW;
    }
    else if (0.0f > st_foc->f4_speed_lpf_rad)
    {
        st_foc->u1_direction = MTR_CCW;
    }
    else
    {
        /* Do nothing */
    }
} /* End of function mtr_decide_direction */

/***********************************************************************************************************************
* Function Name : mtr_angle_speed
* Description   : Angle and speed detection
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_angle_speed(mtr_foc_control_t *st_foc)
{
    float f4_angle_rad;

    /* Estimate voltage disturbance */
    mtr_bemf_observer(&st_foc->st_bemf_obs, st_foc->st_cc.f4_vd_ref, st_foc->st_cc.f4_vq_ref,
                      st_foc->st_cc.f4_id_ad, st_foc->st_cc.f4_iq_ad);

    /* Calculate BEMF */
    st_foc->f4_ed = mtr_bemf_calc_d(&st_foc->st_bemf_obs, st_foc->st_sc.f4_speed_rad, st_foc->st_cc.f4_iq_ad);
    st_foc->f4_eq = mtr_bemf_calc_q(&st_foc->st_bemf_obs, st_foc->st_sc.f4_speed_rad, st_foc->st_cc.f4_id_ad);

    /* Calculate phase error */
    st_foc->f4_phase_err_rad = atanf(st_foc->f4_ed / st_foc->f4_eq);

    /* Prevent phase error becoming NAN when eq is 0 */
    if(isnan(st_foc->f4_phase_err_rad))
    {
        st_foc->f4_phase_err_rad = 0.0f;
    }

    if ((MTR_CONTROL_SPEED | MTR_CONTROL_CURRENT) == st_foc->u2_ctrl_conf)
    {
        /* Estimate angle and speed */
        mtr_angle_speed_pll(&st_foc->st_pll_est, st_foc->f4_phase_err_rad, &st_foc->st_sc.f4_speed_rad);
    }
    else if (MTR_CONTROL_CURRENT == st_foc->u2_ctrl_conf)
    {
        if (MTR_FLG_SET == st_foc->u1_openloop_damping)
        {
            st_foc->st_sc.f4_speed_rad = st_foc->st_sc.f4_ref_speed_rad_ctrl - st_foc->f4_damp_comp_speed;
        }
        else
        {
            st_foc->st_sc.f4_speed_rad = st_foc->st_sc.f4_ref_speed_rad_ctrl;
        }
    }
    else
    {
        /* Do nothing */
    }

    f4_angle_rad = mtr_rotor_angle_get_angle(&st_foc->st_rotor_angle);
    f4_angle_rad += (st_foc->st_sc.f4_speed_rad * st_foc->st_cc.f4_ctrl_period);
    if (f4_angle_rad >= MTR_TWOPI)
    {
        f4_angle_rad= f4_angle_rad - MTR_TWOPI;
    }
    else if (f4_angle_rad < 0.0f)
    {
        f4_angle_rad = f4_angle_rad + MTR_TWOPI;
    }
    else
    {
        /* Do nothing */
    }
    mtr_rotor_angle_update(&st_foc->st_rotor_angle, f4_angle_rad);
} /* End of function mtr_angle_speed */

/***********************************************************************************************************************
* Function Name : mtr_set_speed_ref
* Description   : Updates the speed reference
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : Speed reference
***********************************************************************************************************************/
float mtr_set_speed_ref(mtr_foc_control_t *st_foc)
{
    float f4_speed_rad_ref_buff;

    switch (st_foc->u1_state_speed_ref)
    {
        case MTR_SPEED_ZERO_CONST:
            f4_speed_rad_ref_buff = 0.0f;
            if (MTR_ID_CONST == st_foc->u1_state_id_ref)
            {
                st_foc->u1_state_speed_ref = MTR_SPEED_CHANGE;
            }
        break;

        case MTR_SPEED_CHANGE:
            f4_speed_rad_ref_buff = mtr_speed_rate_limit(&st_foc->st_sc);
        break;

        default:
            /* Do noting */
        break;
    }

    /* Speed reference limit */
    f4_speed_rad_ref_buff = R_MTR_LimitfAbs(f4_speed_rad_ref_buff, st_foc->st_sc.f4_max_speed_rad);

    /* Return speed reference */
    return (f4_speed_rad_ref_buff);
} /* End of function mtr_set_speed_ref */

/***********************************************************************************************************************
* Function Name : mtr_set_iq_ref
* Description   : Updates the q-axis current reference
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : Iq reference
***********************************************************************************************************************/
float mtr_set_iq_ref(mtr_foc_control_t *st_foc)
{
    float f4_iq_ref_buff;
    float f4_temp0;
    uint8_t u1_temp0;

    if (MTR_FLG_SET == st_foc->u1_flag_down_to_ol)
    {
        st_foc->u1_state_iq_ref = MTR_IQ_DOWN;
        st_foc->f4_ol_iq_down_step = st_foc->f4_ol_iq_down_step_ratio * st_foc->st_cc.f4_iq_ref;
    }

    switch (st_foc->u1_state_iq_ref)
    {
        case MTR_IQ_ZERO_CONST:
            /* Iq command will always be zero in this state */
            f4_iq_ref_buff = 0.0f;

            f4_temp0 = fabsf(st_foc->st_sc.f4_ref_speed_rad_ctrl);
            if (MTR_FLG_SET == st_foc->u1_openloop_damping)
            {
                /* Open loop damping control*/
                st_foc->f4_damp_comp_speed =
                    mtr_opl_damp_ctrl(&st_foc->st_opl_damp, st_foc->f4_ed, st_foc->st_sc.f4_ref_speed_rad_ctrl);
            }
            if (f4_temp0 >= st_foc->f4_id_down_speed_rad)
            {
                /* State transient to next state */
                if (MTR_FLG_SET == st_foc->u1_less_switch)
                {
                    st_foc->u1_state_iq_ref = MTR_IQ_AUTO_ADJ;
                    mtr_foc_opl2less_preprocess(st_foc);
                }
                else
                {
                    st_foc->u1_state_iq_ref = MTR_IQ_SPEED_PI_OUTPUT;
                    /* Set the integrator of PLL to the latest speed reference */
                    st_foc->st_pll_est.f4_i_est_speed = st_foc->st_sc.f4_ref_speed_rad_ctrl;
                }
                if (MTR_FLG_SET == st_foc->u1_openloop_damping)
                {
                    /* Open-loop damping control reset */
                    mtr_opl_damp_reset(&st_foc->st_opl_damp);
                    st_foc->f4_damp_comp_speed = 0.0f;
                }
            }

            break;

        case MTR_IQ_AUTO_ADJ:
            /* Iq auto adjustment */
            f4_iq_ref_buff = mtr_opl2less_iq_calc(st_foc->f4_ed, st_foc->f4_eq, st_foc->st_cc.f4_id_ref,
                                                  st_foc->f4_opl_torque_current, st_foc->f4_phase_err_rad);

            /* Limit Iq reference */
            f4_iq_ref_buff = R_MTR_LimitfAbs(f4_iq_ref_buff, st_foc->st_cc.f4_lim_iq);

            if (MTR_CW == st_foc->u1_direction)
            {
                u1_temp0 = (st_foc->f4_phase_err_rad <= st_foc->f4_switch_phase_err_rad);
            }
            else if (MTR_CCW == st_foc->u1_direction)
            {
                u1_temp0 = (st_foc->f4_phase_err_rad >= (-st_foc->f4_switch_phase_err_rad));
            }
            else
            {
                /* Do nothing */
            }

            if (1 == u1_temp0)
            {
                st_foc->u1_state_iq_ref = MTR_IQ_SPEED_PI_OUTPUT;
                st_foc->f4_init_phase_err_rad = 0.0f;

                /* Preset for suppress speed hunting */
                st_foc->st_sc.f4_ref_speed_rad_ctrl = st_foc->f4_speed_lpf_rad;
                st_foc->st_pll_est.f4_i_est_speed = st_foc->st_sc.f4_ref_speed_rad_ctrl;
                st_foc->st_sc.st_pi_speed.f4_refi = f4_iq_ref_buff;
            }
        break;

        case MTR_IQ_SPEED_PI_OUTPUT:
            /* Speed PI control */
            f4_iq_ref_buff = mtr_speed_pi_control(&st_foc->st_sc, st_foc->f4_speed_lpf_rad);

            /* Iq reference limit */
            f4_iq_ref_buff = R_MTR_LimitfAbs(f4_iq_ref_buff, st_foc->st_cc.f4_lim_iq);
        break;

        case MTR_IQ_DOWN:
            f4_iq_ref_buff = st_foc->st_cc.f4_iq_ref - st_foc->f4_ol_iq_down_step;
            f4_temp0 = st_foc->f4_ol_iq_down_step * f4_iq_ref_buff;                 /* Check sign */
            if (f4_temp0 <= 0.0f)
            {
                f4_iq_ref_buff = 0.0f;
                st_foc->u1_state_iq_ref = MTR_IQ_ZERO_CONST;
            }
        break;

        default:
            /* Do noting */
        break;
    }

    /* Return iq reference */
    return (f4_iq_ref_buff);
} /* End of function mtr_set_iq_ref */

/***********************************************************************************************************************
* Function Name : mtr_set_id_ref
* Description   : Updates the d-axis current reference
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : Id reference
***********************************************************************************************************************/
float mtr_set_id_ref(mtr_foc_control_t *st_foc)
{
    float f4_temp0;
    float f4_id_ref_buff;

    if (MTR_FLG_SET == st_foc->u1_flag_down_to_ol)
    {
        st_foc->u1_state_id_ref = MTR_ID_UP;
    }

    if (MTR_FLG_SET == st_foc->u1_flux_weakening)
    {
        /* Flux-weakening process should be ignored unless d-axis current command in certain state */
        st_foc->u1_enable_fluxwkn_flag = 0;
    }

    switch (st_foc->u1_state_id_ref)
    {
        case MTR_ID_UP:
            f4_id_ref_buff = st_foc->st_cc.f4_id_ref + st_foc->f4_ol_id_up_step;
            if (f4_id_ref_buff >= st_foc->f4_ol_id_ref)
            {
                f4_id_ref_buff = st_foc->f4_ol_id_ref;
                st_foc->u1_state_id_ref = MTR_ID_CONST;
            }
        break;

        case MTR_ID_CONST:
            f4_id_ref_buff = st_foc->st_cc.f4_id_ref;
            f4_temp0 = fabsf(st_foc->st_sc.f4_ref_speed_rad_ctrl);
            if (f4_temp0 >= st_foc->f4_id_down_speed_rad)
            {
                st_foc->u1_state_id_ref = MTR_ID_DOWN;
            }
        break;

        case MTR_ID_DOWN:
            f4_id_ref_buff = st_foc->st_cc.f4_id_ref - st_foc->f4_ol_id_down_step;
            if (0 >= f4_id_ref_buff)
            {
                f4_id_ref_buff = 0.0f;
                st_foc->u1_state_id_ref = MTR_ID_ZERO_CONST;
            }
        break;

        case MTR_ID_ZERO_CONST:
        case MTR_ID_FLUXWKN:
            f4_id_ref_buff = 0.0f;
            if (MTR_FLG_SET == st_foc->u1_flux_weakening)
            {
                /* Enables the Flux-weakening when the open-loop has completely ended */
                st_foc->u1_enable_fluxwkn_flag = 1;
                if(FLUXWKN_STATE_BYPASSED != R_FLUXWKN_GetStatus(&st_foc->st_fluxwkn))
                {
                    st_foc->u1_state_id_ref = MTR_ID_FLUXWKN;
                }
                else
                {
                    st_foc->u1_state_id_ref = MTR_ID_ZERO_CONST;
                }
            }
        break;

        default:
            /* Do nothing */
        break;
    }

    /* Return id reference */
    return (f4_id_ref_buff);
} /* End of function mtr_set_id_ref */

/***********************************************************************************************************************
* Function Name : mtr_check_over_speed_error
* Description   : Checks over-speed error
* Arguments     : f4_speed_rad - The electrical speed[rad/s]
*                 f4_speed_limit_rad - The speed[rad/s] threshold of the over-speed error, should be a positive value
* Return Value  : The over-speed error flag
***********************************************************************************************************************/
static inline uint16_t mtr_check_over_speed_error(float f4_speed_rad, float f4_speed_limit_rad)
{
    float f4_temp0;
    uint16_t u2_temp0;

    u2_temp0 = MTR_ERROR_NONE;

    f4_temp0 = fabsf(f4_speed_rad);
    if (f4_temp0 > f4_speed_limit_rad)
    {
        u2_temp0 = MTR_ERROR_OVER_SPEED;                             /* Over speed error */
    }

    return (u2_temp0);
} /* End of function mtr_check_over_speed_error */

/***********************************************************************************************************************
* Function Name : mtr_check_over_voltage_error
* Description   : Checks over-voltage error
* Arguments     : f4_vdc - The actual Vdc value [V]
                  f4_overvoltage_limit - The threshold voltage[V] of over-voltage error
* Return Value  : The flag of over-voltage error
***********************************************************************************************************************/
static inline uint16_t mtr_check_over_voltage_error(float f4_vdc, float f4_overvoltage_limit)
{
    uint16_t u2_temp0;

    u2_temp0 = MTR_ERROR_NONE;

    if (f4_vdc > f4_overvoltage_limit)
    {
        u2_temp0 = MTR_ERROR_OVER_VOLTAGE;                           /* Over voltage error */
    }

    return (u2_temp0);
} /* End of function mtr_check_over_voltage_error */

/***********************************************************************************************************************
* Function Name : mtr_check_under_voltage_error
* Description   : Checks under-voltage error
* Arguments     : f4_vdc - The actual Vdc value [V]
                  f4_undervoltage_limit - The threshold voltage[V] of under-voltage error
* Return Value  : The flag of under-voltage error
***********************************************************************************************************************/
static inline uint16_t mtr_check_under_voltage_error(float f4_vdc, float f4_undervoltage_limit)
{
    uint16_t u2_temp0;

    u2_temp0 = MTR_ERROR_NONE;

    if (f4_vdc < f4_undervoltage_limit)
    {
        u2_temp0 = MTR_ERROR_UNDER_VOLTAGE;                          /* Under voltage error */
    }

    return (u2_temp0);
} /* End of function mtr_check_under_voltage_error */

/***********************************************************************************************************************
* Function Name : mtr_check_over_current_error
* Description   : Checks over-current error
* Arguments     : f4_iu - The actual U-phase current value [A]
                  f4_iv - The actual V-phase current value [A]
                  f4_iw - The actual W-phase current value [A]
                  f4_overcurrent_limit - The threshold current[A] of over-current error
* Return Value  : The flag of over-current error
***********************************************************************************************************************/
static inline uint16_t mtr_check_over_current_error(float f4_iu, float f4_iv, float f4_iw, float f4_overcurrent_limit)
{
    float f4_temp0;
    uint16_t u2_temp0;

    u2_temp0 = MTR_ERROR_NONE;                                       /* Initialize */

    f4_temp0 = fabsf(f4_iu);
    u2_temp0 |= (f4_temp0 > f4_overcurrent_limit);
    f4_temp0 = fabsf(f4_iv);
    u2_temp0 |= (f4_temp0 > f4_overcurrent_limit);
    f4_temp0 = fabsf(f4_iw);
    u2_temp0 |= (f4_temp0 > f4_overcurrent_limit);

    u2_temp0 *= MTR_ERROR_OVER_CURRENT_SW;

    return (u2_temp0);
} /* End of function mtr_check_over_current_error */

/***********************************************************************************************************************
* Function Name : mtr_error_check
* Description   : Checks the errors
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_error_check(mtr_foc_control_t *st_foc)
{
    uint16_t u2_error_flags = 0;

    /*==================================*/
    /* Over current error check     */
    /*==================================*/
    u2_error_flags |= mtr_check_over_current_error(st_foc->f4_iu_ad,
                                                   st_foc->f4_iv_ad,
                                                   st_foc->f4_iw_ad,
                                                   st_foc->f4_overcurrent_limit);

    /*==================================*/
    /* Over voltage error check         */
    /*==================================*/
    u2_error_flags |= mtr_check_over_voltage_error(st_foc->f4_vdc_ad,
                                                   st_foc->f4_overvoltage_limit);

    /*===================================*/
    /* Under voltage error check         */
    /*===================================*/
    if (MTR_FLG_SET == st_foc->u1_flag_charge_cap)    /* Check "capacitor charge completed" */
    {
        u2_error_flags |= mtr_check_under_voltage_error(st_foc->f4_vdc_ad, st_foc->f4_undervoltage_limit);
    }

    /*================================*/
    /* Over speed error check     */
    /*================================*/
    u2_error_flags |= mtr_check_over_speed_error(st_foc->st_sc.f4_speed_rad, st_foc->f4_overspeed_limit_rad);

    if (MTR_ERROR_NONE != u2_error_flags)
    {
        mtr_foc_error(st_foc, u2_error_flags);              /* Execute error event */
    }
} /* End of function mtr_error_check */

/***********************************************************************************************************************
* Function Name : mtr_over_current
* Description   : POE interrupt
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_over_current(mtr_foc_control_t *st_foc)
{
    mtr_foc_error(st_foc, MTR_ERROR_OVER_CURRENT_HW);

    mtr_clear_oc_flag();                              /* Clear forced cutoff flag (detect over current) */
} /* End of function mtr_over_current */

/***********************************************************************************************************************
* Function Name : mtr_foc_phase_err_lpf_setup
* Description   : Setups LPF for estimated phase error by given cutoff frequency[Hz]
* Arguments     : st_foc - The pointer to the FOC structure
*                 f4_phase_err_lpf_cut_freq - Cut off frequency[Hz] of phase error LPF
* Return Value  : None
***********************************************************************************************************************/
void mtr_foc_phase_err_lpf_setup(mtr_foc_control_t *st_foc, float f4_phase_err_lpf_cut_freq)
{
    R_MTR_FirstOrderLpffGainCalc(
        &st_foc->st_phase_err_lpf,
        f4_phase_err_lpf_cut_freq,
        st_foc->st_sc.f4_speed_ctrl_period
    );
} /* End of function mtr_foc_phase_err_lpf_setup */

/***********************************************************************************************************************
* Function Name : mtr_foc_voltage_limit
* Description   : Limit voltage vector, d-axis voltage has higher priority than q-axis voltage
* Arguments     : p_foc - The pointer to the FOC structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_foc_voltage_limit(mtr_foc_control_t *p_foc)
{
    float f4_va_max;
    float f4_vq_lim;
    float f4_vd_ref;

    f4_va_max = R_MOD_GetVamax(&p_foc->st_mod);
    f4_vd_ref = p_foc->st_cc.f4_vd_ref;

    /* The d-axis voltage has high priority than the q-axis */
    /* If absolute value of D-axis voltage exceeds maximum voltage, limit it to the maximum voltage */
    if (f4_vd_ref > f4_va_max)
    {
        p_foc->st_cc.f4_vd_ref = f4_va_max;
        p_foc->st_cc.f4_vq_ref = 0.0f;
    }
    else if (f4_vd_ref < (-f4_va_max))
    {
        p_foc->st_cc.f4_vd_ref = -f4_va_max;
        p_foc->st_cc.f4_vq_ref = 0.0f;
    }
    else
    {
        f4_vq_lim = sqrtf((f4_va_max * f4_va_max) - (f4_vd_ref *  f4_vd_ref));
        p_foc->st_cc.f4_vq_ref = R_MTR_LimitfAbs(p_foc->st_cc.f4_vq_ref, f4_vq_lim);
    }
} /* End of function mtr_foc_voltage_limit */

/***********************************************************************************************************************
* Function Name : mtr_foc_set_nominal_current
* Description   : Setup current related limit values by the nominal current of motor
* Arguments     : st_cc - The pointer to the FOC current control structure
*                 f4_nominal_current_rms - The nominal current[Arms] of motor
* Return Value  : None
***********************************************************************************************************************/
void mtr_foc_set_nominal_current(mtr_foc_control_t *st_foc, float f4_nominal_current_rms)
{
    float f4_ia_max;

    f4_ia_max = f4_nominal_current_rms * MTR_SQRT_3;
    st_foc->f4_overcurrent_limit = (f4_nominal_current_rms * MTR_SQRT_2) * MTR_OVERCURRENT_MARGIN_MULT;
    if(st_foc->f4_overcurrent_limit > IP_CURRENT_LIMIT)
    {
        st_foc->f4_overcurrent_limit = IP_CURRENT_LIMIT;
    }
    st_foc->st_sc.st_pi_speed.f4_ilimit = f4_ia_max;
    st_foc->st_cc.f4_lim_iq = f4_ia_max;
    R_FLUXWKN_SetIamax(&st_foc->st_fluxwkn, f4_ia_max);
} /* End of function mtr_foc_set_nominal_current */

/***********************************************************************************************************************
* Function Name: mtr_foc_opl2less_preprocess
* Description  : Preparations before switching from open-loop to normal FOC
* Arguments    : st_foc - The pointer to the FOC data structure
* Return Value : None
***********************************************************************************************************************/
void mtr_foc_opl2less_preprocess(mtr_foc_control_t *st_foc)
{
    st_foc->f4_opl_torque_current =  mtr_opl2less_torque_current_calc(
        &st_foc->st_motor,
        st_foc->f4_opl2less_sw_time,
        st_foc->f4_ol_id_ref,
        st_foc->f4_phase_err_rad_lpf
    );
} /* End of function mtr_foc_opl2less_preprocess() */
