/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
* EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
* SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS
* SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2018 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name   : r_mtr_foc_control_less_speed.h
* Description : Definitions of sensor-less FOC
***********************************************************************************************************************/
/**********************************************************************************************************************
* History : DD.MM.YYYY Version
*         : 28.09.2018 1.00
***********************************************************************************************************************/

/* Guard against multiple inclusion */
#ifndef R_MTR_FOC_CONTROL_H
#define R_MTR_FOC_CONTROL_H

/***********************************************************************************************************************
* Includes <System Includes> , "Project Includes"
***********************************************************************************************************************/
#include <stdint.h>
#include "r_mtr_config.h"
#include "r_mtr_bemf_observer.h"
#include "r_mtr_filter.h"
#include "r_mtr_fluxwkn.h"
#include "r_mtr_foc_current.h"
#include "r_mtr_foc_speed.h"
#include "r_mtr_mod.h"
#include "r_mtr_opl2less.h"
#include "r_mtr_opl_damp_ctrl.h"
#include "r_mtr_parameter.h"
#include "r_mtr_pi_control.h"
#include "r_mtr_statemachine.h"
#include "r_mtr_transform.h"
#include "r_mtr_volt_err_comp.h"

/***********************************************************************************************************************
* Macro definitions
***********************************************************************************************************************/
/* Id reference status */
#define     MTR_ID_UP                    (0)
#define     MTR_ID_CONST                 (1)
#define     MTR_ID_DOWN                  (2)
#define     MTR_ID_ZERO_CONST            (3)
#define     MTR_ID_FLUXWKN               (4)

/* Iq reference status */
#define     MTR_IQ_ZERO_CONST            (0)
#define     MTR_IQ_SPEED_PI_OUTPUT       (1)
#define     MTR_IQ_AUTO_ADJ              (2)
#define     MTR_IQ_DOWN                  (3)
/* Speed reference status */
#define     MTR_SPEED_ZERO_CONST         (0)
#define     MTR_POS_CONTROL_OUTPUT       (1)
#define     MTR_SPEED_CHANGE             (2)
/* Angle adjust mode */
#define     MTR_ANGLE_ADJ_EXCIT          (0)
#define     MTR_ANGLE_ADJ_HALL           (1)
/* Position reference mode */
#define     MTR_POS_CONST                (0)
#define     MTR_POS_STEP                 (1)
#define     MTR_POS_TRAPEZOID            (2)
/* Position and speed calculation mode */
#define     MTR_PS_CALC_ENCD_INTERRUPT   (0)
#define     MTR_PS_CALC_CTRL_INTERRUPT   (1)
/* Encoder angle adjustment reference status */
#define     MTR_ENCD_ANGLE_ADJ_90DEG     (0)
#define     MTR_ENCD_ANGLE_ADJ_0DEG      (1)
#define     MTR_ENCD_ANGLE_ADJ_FIN       (2)
/* Control loop mode */
#define     MTR_LOOP_SPEED               (0)
#define     MTR_LOOP_POSITION            (1)
/* Ctrl mode */
#define     MTR_CTRL_PID                 (0)
#define     MTR_CTRL_IPD                 (1)
/* Gain mode */
#define     MTR_GAIN_DESIGN_MODE         (0)
#define     MTR_GAIN_DIRECT_MODE         (1)

/***********************************************************************************************************************
* Macro definitions for sequence control
***********************************************************************************************************************/
/* Run mode */
#define     MTR_MODE_INIT               (0x00)
#define     MTR_MODE_BOOT               (0x01)
#define     MTR_MODE_DRIVE              (0x02)
#define     MTR_MODE_ANALYSIS           (0x03)
#define     MTR_MODE_TUNE               (0x04)

/* Control configuration */
#define     MTR_CONTROL_CURRENT         (0x01)
#define     MTR_CONTROL_SPEED           (0x02)
#define     MTR_CONTROL_POSITION        (0x04)
#define     MTR_CONTROL_TORQUE          (0x08)
#define     MTR_CONTROL_VOLTAGE         (0x10)

/* Error status */
#define     MTR_ERROR_NONE              (0x0000)
#define     MTR_ERROR_OVER_CURRENT_HW   (0x0001)
#define     MTR_ERROR_OVER_VOLTAGE      (0x0002)
#define     MTR_ERROR_OVER_SPEED        (0x0004)
#define     MTR_ERROR_HALL_TIMEOUT      (0x0008)
#define     MTR_ERROR_BEMF_TIMEOUT      (0x0010)
#define     MTR_ERROR_HALL_PATTERN      (0x0020)
#define     MTR_ERROR_BEMF_PATTERN      (0x0040)
#define     MTR_ERROR_UNDER_VOLTAGE     (0x0080)
#define     MTR_ERROR_OVER_CURRENT_SW   (0x0100)
#define     MTR_ERROR_UNKNOWN           (0xFFFF)

/* The PWM period[us] */
#define     MTR_PWM_PERIOD_US           (1000.0f / MTR_CARRIER_FREQ)

/* The duty cycle of dead-time */
#define     MTR_DUTY_CYCLE_DEADTIME     (MTR_DEADTIME / MTR_PWM_PERIOD_US)

/* The maximum duty cycle to ensure the time for current sampling */
#define     MTR_MAX_DUTY_CYCLE          (1.0f - (MTR_DUTY_CYCLE_DEADTIME  + (MTR_AD_SAMPLING_TIME / MTR_PWM_PERIOD_US)))

/* The minimum duty cycle to prevent linearity error of PWM output */
#define     MTR_MIN_DUTY_CYCLE          (MTR_DUTY_CYCLE_DEADTIME)

/***********************************************************************************************************************
* Global structure
***********************************************************************************************************************/
typedef struct
{
    uint8_t       u1_id;                        /* Motor id */
    uint16_t      u2_error_status;              /* FOC error flags */
    uint16_t      u2_run_mode;                  /* Run mode */
    uint16_t      u2_ctrl_conf;                 /* Control configuration */
    uint8_t       u1_state_id_ref;              /* The d-axis current command status */
    uint8_t       u1_state_iq_ref;              /* The q-axis current command status */
    uint8_t       u1_state_speed_ref;           /* The speed command status */
    uint8_t       u1_flag_offset_calc;          /* The flag represents that the offset measurement is finished */
    uint8_t       u1_flag_charge_cap;           /* Capacitor charge completed flag */
    uint8_t       u1_direction;                 /* Reference rotation direction */
    uint16_t      u2_offset_calc_time;          /* Calculation time for current offset */
    uint16_t      u2_cnt_adjust;                /* The number of samples to be acquired in offset measurement */
    uint8_t       u1_less_switch;               /* Soft switching */
    uint8_t       u1_flux_weakening;            /* Flux-weakening */
    uint8_t       u1_openloop_damping;          /* Open-loop damping control */
    float         f4_offset_iu;                 /* U-phase current offset value [A] */
    float         f4_offset_iw;                 /* W-phase current offset value [A] */
    float         f4_sum_iu_ad;                 /* U-phase current summation value for offset calculation [A] */
    float         f4_sum_iw_ad;                 /* W-phase current summation value for offset calculation [A] */
    float         f4_vdc_ad;                    /* Bus voltage [V] */
    float         f4_refu;                      /* U-phase reference voltage value [V] */
    float         f4_refv;                      /* V-phase reference voltage value [V] */
    float         f4_refw;                      /* W-phase reference voltage value [V] */
    float         f4_iu_ad;                     /* U-phase current value [A] */
    float         f4_iv_ad;                     /* V-phase current value [A] */
    float         f4_iw_ad;                     /* W-phase current value [A] */
    float         f4_modu;                      /* U-phase duty cycle */
    float         f4_modv;                      /* V-phase duty cycle */
    float         f4_modw;                      /* W-phase duty cycle */
    float         f4_overcurrent_limit;         /* Over-current limit [A] */
    float         f4_overvoltage_limit;         /* Over-voltage limit [V] */
    float         f4_overspeed_limit_rad;       /* Over-speed limit [rad/s] */
    float         f4_undervoltage_limit;        /* Under-voltage limit [V]*/
    float         f4_speed_lpf_rad;             /* Speed processed by LPF */

    /* BEMF observer related members */
    float         f4_ed;                        /* Estimated d-axis component[V] of flux due to the permanent magnet */
    float         f4_eq;                        /* Estimated q-axis component[V] of flux due to the permanent magnet */
    float         f4_phase_err_rad;             /* Estimated phase error[rad] from actual rotor electrical angle */

    /* Open-loop damping related members */
    float         f4_phase_err_rad_lpf;         /* LPF value of phase error[rad] */
    float         f4_init_phase_err_rad;        /* Initialization value of phase error rate */
    float         f4_opl2less_sw_time;          /* Time to switch open-loop to sensor-less [s] */
    float         f4_opl_torque_current;        /* The torque current (Iq) in sensor-less switching */
    float         f4_switch_phase_err_rad;      /* Phase error to decide sensor-less switch timing [rad] */

    /* Startup configurations */
    uint8_t       u1_flag_down_to_ol;           /* The open-loop drive flag */
    float         f4_ol_id_up_step;             /* The d-axis current reference ramping up rate [A/ms] */
    float         f4_ol_id_down_step;           /* The d-axis current reference ramping down rate [A/ms] */
    float         f4_ol_iq_down_step_ratio;     /* The Iq reference ramping down proportion to Iqref before open-loop */
    float         f4_ol_iq_down_step;           /* The q-axis current reference ramping down rate [A/ms] */
    float         f4_ol_id_ref;                 /* The d-axis current reference in open-loop drive */
    float         f4_id_down_speed_rad;         /* The speed threshold[rad/s] to ramp down the d-axis current */
    float         f4_id_up_speed_rad;           /* The speed threshold[rad/s] to ramp up the d-axis current */

    /* Sub-modules */
    mod_t                      st_mod;          /* Modulation structure */
    mtr_rotor_angle_t          st_rotor_angle;  /* The rotor angle and its sine and cosine values */
    mtr_statemachine_t         st_stm;          /* Action structure */
    mtr_parameter_t            st_motor;        /* Motor parameters structure */
    mtr_current_control_t      st_cc;           /* Current control structure */
    mtr_speed_control_t        st_sc;           /* Speed control structure */
    mtr_bemf_observer_t        st_bemf_obs;     /* BEMF observer structure */
    mtr_pll_est_t              st_pll_est;      /* PLL estimator structure */
    mtr_1st_order_lpf_t        st_phase_err_lpf;/* Phase error LPF structure */
    mtr_1st_order_lpf_t        st_est_speed_lpf;/* LPF for estimated speed */

    mtr_volt_comp_t            st_volt_comp_p;  /* Compensates voltage error structure */
    mtr_opl_damp_t             st_opl_damp;     /* Open loop damping control structure */
    float                      f4_damp_comp_speed;
    /* Defines whether use the output from flux-weakening */
    uint8_t                    u1_enable_fluxwkn_flag;

    /* Flux-weakening structure */
    fluxwkn_t                  st_fluxwkn;
} mtr_foc_control_t;

/***********************************************************************************************************************
* Interrupt handler function declarations (defined in the r_mtr_interrupt_xxx.c source file)
***********************************************************************************************************************/
/***********************************************************************************************************************
* Function Name : mtr_foc_interrupt_500us
* Description   : 500us interrupt handler of FOC control
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_foc_interrupt_500us(mtr_foc_control_t *st_foc);

/***********************************************************************************************************************
* Function Name : mtr_foc_interrupt_carrier
* Description   : The carrier interrupt handler of FOC control
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_foc_interrupt_carrier(mtr_foc_control_t *st_foc);

/***********************************************************************************************************************
* Global function definitions
***********************************************************************************************************************/
/***********************************************************************************************************************
* Function Name : mtr_foc_init
* Description   : Initializes FOC drive system with default configuration and specified motor ID
* Arguments     : st_foc - The pointer to the FOC data structure
*                 u1_id - The motor ID which specify the motor connectors
* Return Value  : None
***********************************************************************************************************************/
void mtr_foc_init(mtr_foc_control_t *st_foc, uint8_t u1_id);

/***********************************************************************************************************************
* Function Name : mtr_foc_motor_default_init
* Description   : Initializes motor drive modules with default configuration
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_foc_motor_default_init(mtr_foc_control_t *st_foc);

/***********************************************************************************************************************
* Function Name : mtr_foc_motor_reset
* Description   : Resets motor drive modules, configurations will not be reset
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_foc_motor_reset(mtr_foc_control_t *st_foc);

/***********************************************************************************************************************
* Function Name : mtr_foc_error
* Description   : Sets error flags and trigger error event when u2_error_flag is not zero
* Arguments     : st_foc - The pointer to the FOC data structure
*                 u2_error_flag - The error flags of the errors to report
* Return Value  : None
***********************************************************************************************************************/
void mtr_foc_error(mtr_foc_control_t *st_foc, uint16_t u2_error_flag);

/***********************************************************************************************************************
* Function Name : mtr_calib_current_offset
* Description   : Measures the current offset
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_calib_current_offset(mtr_foc_control_t *st_foc);

/***********************************************************************************************************************
* Function Name : mtr_current_offset_adjustment
* Description   : Calibrates the current offset
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_current_offset_adjustment(mtr_foc_control_t *st_foc);

/***********************************************************************************************************************
* Function Name : mtr_decide_direction
* Description   : Updates direction by sign of electrical speed
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_decide_direction(mtr_foc_control_t *st_foc);

/***********************************************************************************************************************
* Function Name : mtr_angle_speed
* Description   : Angle and speed detection
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_angle_speed(mtr_foc_control_t *st_foc);

/***********************************************************************************************************************
* Function Name : mtr_set_speed_ref
* Description   : Updates the speed reference
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : Speed reference
***********************************************************************************************************************/
float mtr_set_speed_ref(mtr_foc_control_t *st_foc);

/***********************************************************************************************************************
* Function Name : mtr_set_iq_ref
* Description   : Updates the q-axis current reference
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : Iq reference
***********************************************************************************************************************/
float mtr_set_iq_ref(mtr_foc_control_t *st_foc);

/***********************************************************************************************************************
* Function Name : mtr_set_id_ref
* Description   : Updates the d-axis current reference
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : Id reference
***********************************************************************************************************************/
float mtr_set_id_ref(mtr_foc_control_t *st_foc);

/***********************************************************************************************************************
* Function Name : mtr_error_check
* Description   : Checks the errors
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_error_check(mtr_foc_control_t *st_foc);

/***********************************************************************************************************************
* Function Name : mtr_over_current
* Description   : POE interrupt
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_over_current(mtr_foc_control_t *st_foc);

/***********************************************************************************************************************
* Function Name : mtr_foc_phase_err_lpf_setup
* Description   : Setups LPF for estimated phase error by given cutoff frequency[Hz]
* Arguments     : st_foc - The pointer to the FOC structure
*               : f4_phase_err_lpf_cut_freq - Cut off frequency[Hz] of phase error LPF
* Return Value  : None
***********************************************************************************************************************/
void mtr_foc_phase_err_lpf_setup(mtr_foc_control_t *st_foc, float f4_phase_err_lpf_cut_freq);

/***********************************************************************************************************************
* Function Name : mtr_foc_voltage_limit
* Description   : Limit voltage vector, d-axis voltage has higher priority than q-axis voltage
* Arguments     : st_foc - The pointer to the FOC structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_foc_voltage_limit(mtr_foc_control_t *p_foc);

/***********************************************************************************************************************
* Function Name : mtr_foc_set_nominal_current
* Description   : Setup current related limit values by the nominal current of motor
* Arguments     : st_foc - The pointer to the FOC current control structure
*               : f4_nominal_current_rms - The nominal current[Arms] of motor
* Return Value  : None
***********************************************************************************************************************/
void mtr_foc_set_nominal_current(mtr_foc_control_t *st_foc, float f4_nominal_current_rms);

/***********************************************************************************************************************
* Function Name: mtr_foc_opl2less_preprocess
* Description  : Preparations before switching from open-loop to normal FOC
* Arguments    : st_foc - The pointer to the FOC data structure
* Return Value : None
***********************************************************************************************************************/
void mtr_foc_opl2less_preprocess(mtr_foc_control_t *st_foc);

#endif /* R_MTR_FOC_CONTROL_H */
