/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
* EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
* SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS
* SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2018 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name   : r_mtr_parameter.h
* Description : Definitions of parameters used in control
***********************************************************************************************************************/
/**********************************************************************************************************************
* History : DD.MM.YYYY Version
*         : 28.09.2018 1.00
***********************************************************************************************************************/

/* Guard against multiple inclusion */
#ifndef R_MTR_PARAMETER_H
#define R_MTR_PARAMETER_H

/***********************************************************************************************************************
* Includes <System Includes> , "Project Includes"
***********************************************************************************************************************/
#include <stdint.h>
#include "r_mtr_common.h"
#include "r_mtr_motor_parameter.h"
#include "r_mtr_control_parameter.h"
#include "r_mtr_inverter_parameter.h"

/***********************************************************************************************************************
* Macro definitions
***********************************************************************************************************************/
/* Motor parameter */
#define     MTR_M                        (MP_MAGNETIC_FLUX)           /* Permanent magnetic flux [Wb] */
#define     MTR_R                        (MP_RESISTANCE)              /* Resistance [ohm] */
#define     MTR_LD                       (MP_D_INDUCTANCE)            /* D-axis inductance [H] */
#define     MTR_LQ                       (MP_Q_INDUCTANCE)            /* Q-axis inductance [H] */
#define     MTR_J                        (MP_ROTOR_INERTIA)           /* Rotor inertia [kgm^2] */
#define     MTR_NOMINAL_CURRENT_RMS      (MP_NOMINAL_CURRENT_RMS)     /* The nominal current[Arms] of motor */
#define     MTR_POLE_PAIRS               (MP_POLE_PAIRS)              /* Pole pairs */
#define     MTR_RPM_RAD                  ((MTR_POLE_PAIRS * MTR_TWOPI) / 60.0f)
                                                                      /* [rpm]->[rad/s] */

/* Design parameter */
#define     MTR_CURRENT_OMEGA            (CP_CURRENT_OMEGA)           /* Natural frequency of current loop */
#define     MTR_CURRENT_ZETA             (CP_CURRENT_ZETA)            /* Damping ratio of current loop */
#define     MTR_SPEED_OMEGA              (CP_SPEED_OMEGA)             /* Natural frequency of speed loop */
#define     MTR_SPEED_ZETA               (CP_SPEED_ZETA)              /* Damping ratio of speed loop */
#define     MTR_E_OBS_OMEGA              (CP_E_OBS_OMEGA)             /* Natural frequency of BEMF observer */
#define     MTR_E_OBS_ZETA               (CP_E_OBS_ZETA)              /* Damping ratio of BEMF observer */
#define     MTR_PLL_EST_OMEGA            (CP_PLL_EST_OMEGA)           /* Natural frequency of PLL Speed estimate loop */
#define     MTR_PLL_EST_ZETA             (CP_PLL_EST_ZETA)            /* Damping ratio of PLL Speed estimate loop */

/****** Macro of current control*****/
#define     MTR_DEADTIME                 (IP_DEADTIME)                /* Deadtime [us] */
#define     MTR_CURRENT_RANGE            (IP_CURRENT_RANGE)           /* For current scaling */
#define     MTR_VDC_RANGE                (IP_VDC_RANGE)               /* For Vdc scaling */
#define     MTR_OFFSET_CALC_TIME         (512)                        /* Current offset calculation time */
#define     MTR_I_LIMIT_VD               (IP_INPUT_V * 0.5f)          /* Current PI integral term limit for vd */
#define     MTR_I_LIMIT_VQ               (IP_INPUT_V * 0.5f)          /* Current PI integral term limit for vq */
#define     MTR_OL_ID_REF                (CP_OL_ID_REF)               /* Id reference when low speed [A] */
#define     MTR_ID_UP_TIME               (256.0f)                     /* Time to increase id [ms] */
#define     MTR_ID_UP_STEP               (MTR_OL_ID_REF / MTR_ID_UP_TIME)
                                                                      /* Addition value of id */
#define     MTR_ID_DOWN_TIME             (50.0f)                      /* Time to decrease id [ms] */
#define     MTR_ID_DOWN_STEP             (MTR_OL_ID_REF / MTR_ID_DOWN_TIME)
                                                                      /* Addition value of id */
#define     MTR_MAX_CURRENT_OMEGA        (1000.0f)                    /* Max natural frequency of current loop [Hz] */
#define     MTR_FREQ_BAND_LIMIT          (3.0f)                       /* Motor control natural frequency limit */
#define     MTR_MAX_SPEED_RPM            (CP_MAX_SPEED_RPM)           /* Max reference speed [rpm] */
#define     MTR_MAX_SPEED_RAD            (MTR_MAX_SPEED_RPM * MTR_TWOPI_60)
                                                                      /* Max reference speed [rad/s] (mechanical) */
#define     MTR_OVERSPEED_LIMIT_RPM      (CP_OVERSPEED_LIMIT_RPM)     /* Over speed limit [rpm] */
#define     MTR_OVERSPEED_LIMIT_RAD      (MTR_OVERSPEED_LIMIT_RPM * MTR_RPM_RAD)
                                                                      /* Over speed limit [rad/s] (electrical) */
#define     MTR_LIMIT_SPEED_CHANGE       (0.5f)                       /* Limit of speed change [rpm/ms] */
#define     MTR_OVERCURRENT_MARGIN_MULT  (1.5f)                       /* Multiplier for over-current limit*/
#define     MTR_OVERCURRENT_LIMIT        (MP_NOMINAL_CURRENT_RMS * MTR_SQRT_2 * MTR_OVERCURRENT_MARGIN_MULT)
                                                                      /* Over current limit [A] */
#define     MTR_OVERVOLTAGE_LIMIT        (IP_OVERVOLTAGE_LIMIT)       /* Over voltage limit [V] */
#define     MTR_UNDERVOLTAGE_LIMIT       (IP_UNDERVOLTAGE_LIMIT)      /* Under voltage limit [V] */
#define     MTR_INPUT_V                  (IP_INPUT_V)                 /* Input DC voltage [V] */
#define     MTR_MCU_ON_V                 (MTR_INPUT_V * 0.8f)         /* MCU power on voltage */

/***** Macro of position & speed control *****/
#define     MTR_ID_DOWN_SPEED_RPM        (CP_ID_DOWN_SPEED_RPM)       /* Speed to start decreasing id [rpm] */
#define     MTR_ID_DOWN_SPEED_RAD        (MTR_ID_DOWN_SPEED_RPM * MTR_RPM_RAD)
                                                                      /* Speed to start decreasing id [rad] */
#define     MTR_ID_UP_SPEED_RPM          (CP_ID_UP_SPEED_RPM)         /* Speed to start increasing id [rpm] */
#define     MTR_ID_UP_SPEED_RAD          (MTR_ID_UP_SPEED_RPM * MTR_RPM_RAD)
                                                                      /* Speed to start increasing id [rad] */
#define     MTR_I_LIMIT_IQ               (MTR_NOMINAL_CURRENT_RMS * MTR_SQRT_3)
                                                                      /* Speed PI integral term limit for iq */
#define     MTR_LIMIT_IQ                 (MTR_NOMINAL_CURRENT_RMS * MTR_SQRT_3)
                                                                      /* Speed PI limit for iq */
#define     MTR_SPEED_CTRL_PERIOD        (0.0005f)                    /* Control period for speed loop */
#define     MTR_EST_SPEED_LPF_OMEGA_Hz   (25.0f)                      /* Cutoff frequency [Hz] of estimated speed LPF */

/***********************************************************************************************************************
* Global structure
***********************************************************************************************************************/
/* Motor parameter structure */
typedef struct
{
    uint16_t    u2_mtr_pp;                        /* Pole pairs */
    float       f4_mtr_r;                         /* Resistance [ohm] */
    float       f4_mtr_ld;                        /* Inductance for d-axis [H] */
    float       f4_mtr_lq;                        /* Inductance for q-axis [H] */
    float       f4_mtr_m;                         /* Magnet flux [Wb] */
    float       f4_mtr_j;                         /* Rotor inertia [kgm^2] */
} mtr_parameter_t;

/* Design parameter structure */
typedef struct
{
    float     f4_current_omega;                   /* Natural frequency[Hz] for current loop gain design */
    float     f4_current_zeta;                    /* Damping ratio for current loop gain design */
    float     f4_speed_omega;                     /* Natural frequency[Hz] for speed loop gain design */
    float     f4_speed_zeta;                      /* Damping ratio for speed loop gain design */
    float     f4_e_obs_omega;                     /* Natural frequency[Hz] for BEMF observer gain design */
    float     f4_e_obs_zeta;                      /* Damping ratio for current loop gain design */
    float     f4_pll_est_omega;                   /* Natural frequency[Hz] for PLL estimator gain design */
    float     f4_pll_est_zeta;                    /* Damping ratio for current loop gain design */
    float     f4_ed_hpf_omega;                    /* Natural frequency[Hz] for HPF in open-loop damping gain design */
    float     f4_ol_damping_zeta;                 /* Damping ratio for open-loop damping gain design */
    float     f4_phase_err_lpf_cut_freq;          /* The cut-off frequency [Hz] of phase error LPF gain design */
} mtr_design_parameter_t;

/* Control gain structure */
typedef struct
{
    float     f4_id_kp;                           /* The P gain of d-axis current PI for direct gain setting */
    float     f4_id_ki;                           /* The I gain of d-axis current PI for direct gain setting */
    float     f4_iq_kp;                           /* The P gain of q-axis current PI for direct gain setting */
    float     f4_iq_ki;                           /* The I gain of q-axis current PI for direct gain setting */
    float     f4_speed_kp;                        /* The P gain of speed PI for direct gain setting */
    float     f4_speed_ki;                        /* The I gain of speed PI for direct gain setting */
} mtr_ctrl_gain_t;

#endif /* R_MTR_PARAMETER_H */
