/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
* EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
* SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS
* SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2018 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name   : r_mtr_driver_access.c
* Description : The processes of accessing driver
***********************************************************************************************************************/
/**********************************************************************************************************************
* History : DD.MM.YYYY Version
*         : 28.09.2018 1.00
***********************************************************************************************************************/

/***********************************************************************************************************************
* Includes <System Includes> , "Project Includes"
***********************************************************************************************************************/
#include "r_mtr_driver_access.h"
#include "r_mtr_common.h"
#include "r_mtr_config.h"
#include "r_mtr_ctrl_mcu.h"
#include "r_mtr_ctrl_mrssk.h"
#include "r_mtr_foc_control_less_speed.h"

/***********************************************************************************************************************
* Private function declarations
***********************************************************************************************************************/
static inline mtr_foc_control_t* get_foc_by_id(uint8_t u1_id);

/***********************************************************************************************************************
* Private variables
***********************************************************************************************************************/
mtr_ctrl_input_t st_ctrl_input_buff;     /* Structure for ICS input */
static uint8_t          u1_trig_enable_write;   /* The trigger for FOC command and configuration update */

/***********************************************************************************************************************
* Exported global variables
***********************************************************************************************************************/
extern mtr_foc_control_t g_st_foc;

/***********************************************************************************************************************
* Function Name : R_MTR_InitControl
* Description   : Initializes FOC motor controller by given motor ID
* Arguments     : u1_id - Motor ID
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_InitControl(uint8_t u1_id)
{
    mtr_foc_control_t *pfoc;

    pfoc = get_foc_by_id(u1_id);
    mtr_foc_init(pfoc, u1_id);       /* Initialize motor ID */
} /* End of function R_MTR_InitControl */

/***********************************************************************************************************************
* Function Name : R_MTR_ExecEvent
* Description   : Execute event of FOC control
* Arguments     : u1_event - the event index
*                 u1_id - Motor ID
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_ExecEvent(uint8_t u1_event, uint8_t u1_id)
{
    mtr_foc_control_t *pfoc;

    pfoc = get_foc_by_id(u1_id);
    mtr_statemachine_event(&pfoc->st_stm, pfoc, u1_event);
} /* End of function R_MTR_ExecEvent */

/***********************************************************************************************************************
* Function Name : R_MTR_ChargeCapacitor
* Description   : Wait for charging capacitor
* Arguments     : u1_id - Motor ID
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_ChargeCapacitor(uint8_t u1_id)
{
    mtr_foc_control_t *pfoc;

    pfoc = get_foc_by_id(u1_id);
    while (MTR_MCU_ON_V > pfoc->f4_vdc_ad)             /* Wait until Vdc exceed MTR_MCU_ON_V */
    {
        pfoc->f4_vdc_ad = mtr_get_vdc(u1_id);
        R_MTR_ClearWdt();                              /* Watch dog timer clear */
    }
    pfoc->u1_flag_charge_cap = MTR_FLG_SET;            /* Flag set "capacitor charge completed" */
} /* End of function R_MTR_ChargeCapacitor */

/***********************************************************************************************************************
* Function Name : R_MTR_SetVariables
* Description   : Set ICS input buffer to motor control structure members
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_SetVariables(void)
{
    /* Motor parameter */
    g_st_foc.st_motor = st_ctrl_input_buff.st_motor;

    R_FLUXWKN_SetMotor(&g_st_foc.st_fluxwkn ,&g_st_foc.st_motor);

    /* [rpm]->[rad/s]*/
    mtr_set_param_rpm_to_rad(&g_st_foc.st_sc, g_st_foc.st_motor.u2_mtr_pp);

    /* Rotation direction */
    mtr_set_param_ref_dir(&g_st_foc.st_sc, (uint8_t)st_ctrl_input_buff.u1_direction);

    /* Reference speed [rpm] */
    mtr_set_param_ref_speed(&g_st_foc.st_sc, (float)st_ctrl_input_buff.s2_ref_speed_rpm);

    /* Limit of speed change */
    mtr_set_param_limit_speed_change(&g_st_foc.st_sc, st_ctrl_input_buff.f4_limit_speed_change);

    /* Max speed */
    mtr_set_param_max_speed(&g_st_foc.st_sc, st_ctrl_input_buff.u2_max_speed_rpm);

    /* Setup nominal current */
    mtr_foc_set_nominal_current(&g_st_foc, st_ctrl_input_buff.f4_nominal_current_rms);

    /* Set over-speed limit */
    g_st_foc.f4_overspeed_limit_rad = st_ctrl_input_buff.u2_overspeed_limit_rpm * g_st_foc.st_sc.f4_rpm_rad;

    /* Id reference when open-loop */
    g_st_foc.f4_ol_id_ref = st_ctrl_input_buff.f4_ref_id;

    /* Addition value of id reference when doing encoder angle alignment */
    g_st_foc.f4_ol_id_up_step = st_ctrl_input_buff.f4_ref_id / st_ctrl_input_buff.f4_id_up_time;

    /* Current offset calculation time */
    g_st_foc.u2_offset_calc_time = st_ctrl_input_buff.u2_offset_calc_time;

    /* Speed to start increasing id [rpm] */
    g_st_foc.f4_id_up_speed_rad = (float)st_ctrl_input_buff.u2_id_up_speed_rpm * g_st_foc.st_sc.f4_rpm_rad;

    /* Addition value of id reference when low speed */
    g_st_foc.f4_ol_id_up_step = st_ctrl_input_buff.f4_ref_id / st_ctrl_input_buff.f4_id_up_time;

    /* Speed to start decreasing id [rpm]*/
    g_st_foc.f4_id_down_speed_rad = (float)st_ctrl_input_buff.u2_id_down_speed_rpm * g_st_foc.st_sc.f4_rpm_rad;

    /* Subtraction value of id */
    g_st_foc.f4_ol_id_down_step = st_ctrl_input_buff.f4_ref_id / st_ctrl_input_buff.f4_id_down_time;

    /* Subtraction ratio of iq when enter open-loop */
    g_st_foc.f4_ol_iq_down_step_ratio = 1.0f / st_ctrl_input_buff.f4_id_up_time;

    g_st_foc.f4_switch_phase_err_rad = st_ctrl_input_buff.f4_switch_phase_err_deg * MTR_TWOPI_360;

    /* Time[s] to switch open-loop to sensor-less */
    g_st_foc.f4_opl2less_sw_time = st_ctrl_input_buff.f4_opl2less_sw_time;

    /* Rate[krpm/s] of reference speed for open-loop damping control feedback speed limiter */
    mtr_opl_damp_set_limit(&g_st_foc.st_opl_damp, st_ctrl_input_buff.f4_ol_damping_fb_limit_rate);

    /* Cut off frequency[Hz] of phase error LPF */
    mtr_foc_phase_err_lpf_setup(&g_st_foc, st_ctrl_input_buff.st_design_params.f4_phase_err_lpf_cut_freq);

    if (MTR_MODE_INACTIVE == mtr_statemachine_get_status(&g_st_foc.st_stm))
    {
        g_st_foc.u1_less_switch = st_ctrl_input_buff.u1_less_switch;
        g_st_foc.u1_flux_weakening = st_ctrl_input_buff.u1_flux_weakening;
        g_st_foc.st_volt_comp_p.u1_volt_err_comp_enable = st_ctrl_input_buff.u1_volt_err_comp;
        g_st_foc.u1_openloop_damping = st_ctrl_input_buff.u1_openloop_damping;
    }

} /* End of function R_MTR_SetVariables */

/***********************************************************************************************************************
* Function Name : R_MTR_SetUserifMode
* Description   : Set User interface mode
* Arguments     : u1_user_mode - user mode (0:Disable auto transfer 1:Enable auto tranfer)
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_SetUserifMode(uint8_t u1_user_mode)
{
    st_ctrl_input_buff.u1_user_mode = u1_user_mode;
} /* End of function R_MTR_SetUserifMode */

/***********************************************************************************************************************
* Function Name : R_MTR_SetSpeed
* Description   : Set speed reference
* Arguments     : ref_speed - speed reference [rpm]
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_SetSpeed(int16_t ref_speed)
{
    st_ctrl_input_buff.s2_ref_speed_rpm = ref_speed;
} /* End of function R_MTR_SetSpeed */

/***********************************************************************************************************************
* Function Name : R_MTR_SetDir
* Description   : Set rotation direction
* Arguments     : dir - rotational direction
*                 u1_id - Motor ID
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_SetDir(uint8_t dir, uint8_t u1_id)
{
    mtr_foc_control_t *pfoc;

    pfoc = get_foc_by_id(u1_id);
    pfoc->st_sc.u1_ref_dir = dir;
} /* End of function R_MTR_SetDir */

/***********************************************************************************************************************
* Function Name : R_MTR_GetSpeed
* Description   : Get speed value
* Arguments     : u1_id - Motor ID
* Return Value  : Speed[rpm]
***********************************************************************************************************************/
int16_t R_MTR_GetSpeed(uint8_t u1_id)
{
    int16_t s2_speed_rpm;
    mtr_foc_control_t *pfoc;

    pfoc = get_foc_by_id(u1_id);
    s2_speed_rpm = (int16_t)(pfoc->st_sc.f4_speed_rad / pfoc->st_sc.f4_rpm_rad);

    return (s2_speed_rpm);
} /* End of function R_MTR_GetSpeed */

/***********************************************************************************************************************
* Function Name : R_MTR_GetDir
* Description   : Get rotation direction
* Arguments     : u1_id - Motor ID
* Return Value  : Rotational direction
***********************************************************************************************************************/
uint8_t R_MTR_GetDir(uint8_t u1_id)
{
    uint8_t u1_direction;
    mtr_foc_control_t *pfoc;

    pfoc = get_foc_by_id(u1_id);
    u1_direction = pfoc->u1_direction;

    return (u1_direction);
} /* End of function R_MTR_GetDir */

/***********************************************************************************************************************
* Function Name : R_MTR_GetStatus
* Description   : Get status of motor control system
* Arguments     : u1_id - Motor ID
* Return Value  : Motor status
***********************************************************************************************************************/
uint8_t R_MTR_GetStatus(uint8_t u1_id)
{
    uint8_t u1_mode_system;
    mtr_foc_control_t *pfoc;

    pfoc = get_foc_by_id(u1_id);
    u1_mode_system = mtr_statemachine_get_status(&pfoc->st_stm);

    return (u1_mode_system);
} /* End of function R_MTR_GetStatus */

/***********************************************************************************************************************
* Function Name : R_MTR_InputBuffParamReset
* Description   : Initialize variables when motor control reset
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_InputBuffParamReset(void)
{
    /* Initialize member of ICS input buffer structure */
    st_ctrl_input_buff.u1_user_mode                               = MTR_DISABLE_AUTO_SET;
    st_ctrl_input_buff.u1_direction                               = MTR_CW;
    st_ctrl_input_buff.s2_ref_speed_rpm                           = 0;
    st_ctrl_input_buff.u2_max_speed_rpm                           = MTR_MAX_SPEED_RPM;
    st_ctrl_input_buff.u2_overspeed_limit_rpm                     = MTR_OVERSPEED_LIMIT_RPM;
    st_ctrl_input_buff.u2_offset_calc_time                        = MTR_OFFSET_CALC_TIME;
    st_ctrl_input_buff.st_motor.u2_mtr_pp                         = MTR_POLE_PAIRS;
    st_ctrl_input_buff.st_motor.f4_mtr_r                          = MTR_R;
    st_ctrl_input_buff.st_motor.f4_mtr_ld                         = MTR_LD;
    st_ctrl_input_buff.st_motor.f4_mtr_lq                         = MTR_LQ;
    st_ctrl_input_buff.st_motor.f4_mtr_m                          = MTR_M;
    st_ctrl_input_buff.st_motor.f4_mtr_j                          = MTR_J;
    st_ctrl_input_buff.f4_ref_id                                  = MTR_OL_ID_REF;
    st_ctrl_input_buff.f4_id_up_time                              = MTR_ID_UP_TIME;
    st_ctrl_input_buff.f4_limit_speed_change                      = MTR_LIMIT_SPEED_CHANGE;
    st_ctrl_input_buff.st_design_params.f4_current_omega          = MTR_CURRENT_OMEGA;
    st_ctrl_input_buff.st_design_params.f4_current_zeta           = MTR_CURRENT_ZETA;
    st_ctrl_input_buff.st_design_params.f4_speed_omega            = MTR_SPEED_OMEGA;
    st_ctrl_input_buff.st_design_params.f4_speed_zeta             = MTR_SPEED_ZETA;
    st_ctrl_input_buff.st_design_params.f4_e_obs_omega            = MTR_E_OBS_OMEGA;
    st_ctrl_input_buff.st_design_params.f4_e_obs_zeta             = MTR_E_OBS_ZETA;
    st_ctrl_input_buff.st_design_params.f4_pll_est_omega          = MTR_PLL_EST_OMEGA;
    st_ctrl_input_buff.st_design_params.f4_pll_est_zeta           = MTR_PLL_EST_ZETA;
    st_ctrl_input_buff.st_ctrl_gain.f4_id_kp                      = 0.0f;
    st_ctrl_input_buff.st_ctrl_gain.f4_id_ki                      = 0.0f;
    st_ctrl_input_buff.st_ctrl_gain.f4_iq_kp                      = 0.0f;
    st_ctrl_input_buff.st_ctrl_gain.f4_iq_ki                      = 0.0f;
    st_ctrl_input_buff.st_ctrl_gain.f4_speed_kp                   = 0.0f;
    st_ctrl_input_buff.st_ctrl_gain.f4_speed_ki                   = 0.0f;
    st_ctrl_input_buff.f4_nominal_current_rms                     = MTR_NOMINAL_CURRENT_RMS;
    st_ctrl_input_buff.u2_id_up_speed_rpm                         = MTR_ID_UP_SPEED_RPM;
    st_ctrl_input_buff.u2_id_down_speed_rpm                       = MTR_ID_DOWN_SPEED_RPM;
    st_ctrl_input_buff.f4_id_down_time                            = MTR_ID_DOWN_TIME;
    st_ctrl_input_buff.f4_switch_phase_err_deg                    = MTR_SWITCH_PHASE_ERR_DEG;
    st_ctrl_input_buff.f4_opl2less_sw_time                        = MTR_OPL2LESS_SW_TIME;
    st_ctrl_input_buff.st_design_params.f4_ed_hpf_omega           = MTR_ED_HPF_OMEGA;
    st_ctrl_input_buff.st_design_params.f4_ol_damping_zeta        = MTR_OL_DAMPING_ZETA;
    st_ctrl_input_buff.f4_ol_damping_fb_limit_rate                = MTR_FB_SPEED_LIMIT_RATE;
    st_ctrl_input_buff.st_design_params.f4_phase_err_lpf_cut_freq = MTR_PHASE_ERR_LPF_CUT_FREQ;

    /* Initialize control gains */
    R_MTR_CtrlGainCalc();

    /* Initialize write enable switch */
    u1_trig_enable_write = 0;
} /* End of function R_MTR_InputBuffParamReset */

/***********************************************************************************************************************
* Function Name : R_MTR_CtrlInput
* Description   : Store com_* variables to ICS input buffer
* Arguments     : st_ctrl_input - The pointer to the control input structure
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_CtrlInput(mtr_ctrl_input_t *st_ctrl_input)
{
    /* Transfer control input value to buffer structure */
    st_ctrl_input_buff = *st_ctrl_input;

    /* Transfer control input value to buffer structure */
    u1_trig_enable_write = MTR_FLG_SET;
} /* End of function R_MTR_CtrlInput */

/***********************************************************************************************************************
* Function Name : R_MTR_CtrlGainCalc
* Description   : motor control parameter & gain calculation
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_CtrlGainCalc(void)
{
#if (GAIN_MODE == MTR_GAIN_DESIGN_MODE)
    /* The dq-axis current PI gains */
    mtr_foc_current_pi_gain_calc(
        &g_st_foc.st_motor,
        &st_ctrl_input_buff.st_design_params,
        &g_st_foc.st_cc,
        g_st_foc.st_cc.f4_ctrl_period
    );

    /* Speed PI gains */
    mtr_speed_pi_gain_calc(&g_st_foc.st_motor, &st_ctrl_input_buff.st_design_params, &g_st_foc.st_sc,
                            g_st_foc.st_sc.f4_speed_ctrl_period);

#elif (GAIN_MODE == MTR_GAIN_DIRECT_MODE)
    /* Write PI gain directly */
    mtr_pi_ctrl_set_kp(&g_st_foc.st_cc.st_pi_id, st_ctrl_input_buff.st_ctrl_gain.f4_id_kp);
    mtr_pi_ctrl_set_ki(&g_st_foc.st_cc.st_pi_id, st_ctrl_input_buff.st_ctrl_gain.f4_id_ki);
    mtr_pi_ctrl_set_kp(&g_st_foc.st_cc.st_pi_iq, st_ctrl_input_buff.st_ctrl_gain.f4_iq_kp);
    mtr_pi_ctrl_set_ki(&g_st_foc.st_cc.st_pi_iq, st_ctrl_input_buff.st_ctrl_gain.f4_iq_ki);
    mtr_pi_ctrl_set_kp(&g_st_foc.st_sc.st_pi_speed, st_ctrl_input_buff.st_ctrl_gain.f4_speed_kp);
    mtr_pi_ctrl_set_ki(&g_st_foc.st_sc.st_pi_speed, st_ctrl_input_buff.st_ctrl_gain.f4_speed_ki);
#endif

    /* BEMF observer gain */
    mtr_bemf_observer_gain_calc(&g_st_foc.st_motor, &st_ctrl_input_buff.st_design_params, &g_st_foc.st_bemf_obs,
                                 g_st_foc.st_cc.f4_ctrl_period);

    /* PLL gain */
    mtr_angle_speed_gain_calc(&st_ctrl_input_buff.st_design_params, &g_st_foc.st_pll_est,
                              g_st_foc.st_cc.f4_ctrl_period);

    /* Open-loop damping control gain */
    mtr_opl_damp_r_set_gain(
        &g_st_foc.st_opl_damp,
        g_st_foc.st_motor.u2_mtr_pp,
        g_st_foc.st_motor.f4_mtr_m,
        g_st_foc.st_motor.f4_mtr_j,
        st_ctrl_input_buff.st_design_params.f4_ol_damping_zeta,
        st_ctrl_input_buff.st_design_params.f4_ed_hpf_omega,
        g_st_foc.f4_ol_id_ref,
        g_st_foc.f4_id_down_speed_rad,
        g_st_foc.st_sc.f4_speed_ctrl_period
    );
} /* End of function R_MTR_CtrlGainCalc */

/***********************************************************************************************************************
* Function Name : R_MTR_UpdatePolling
* Description   : Polling u1_trig_enable_write to update configurations and commands
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_UpdatePolling(void)
{
    if (MTR_FLG_SET == u1_trig_enable_write)
    {
        R_MTR_SetVariables();
        R_MTR_CtrlGainCalc();
        u1_trig_enable_write = MTR_FLG_CLR;
    }
} /* End of function R_MTR_UpdatePolling */

/***********************************************************************************************************************
* Function Name : get_foc_by_id
* Description   : Gets the pointer of FOC structure corresponding to the given motor ID
* Arguments     : u1_id - Motor ID
* Return Value  : Foc control structure pointer
***********************************************************************************************************************/
static inline mtr_foc_control_t* get_foc_by_id(uint8_t u1_id)
{
    switch (u1_id)
    {
        case MTR_ID_A:
            return (&g_st_foc);
        break;

        default:
            return 0;
        break;
    }
} /* End of function get_foc_by_id */

/***********************************************************************************************************************
* Function Name : R_MTR_AutoSetVariables
* Description   : Set control input buffer to motor control structure members
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_AutoSetVariables(void)
{
    mtr_foc_control_t *pfoc = get_foc_by_id(MTR_ID_A);

    if(MTR_ENABLE_AUTO_SET == st_ctrl_input_buff.u1_user_mode)
    {
        /* reference speed */
        mtr_set_param_ref_speed(&pfoc->st_sc, st_ctrl_input_buff.s2_ref_speed_rpm);
    }
} /* End of function R_MTR_AutoSetVariables */
