/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
* EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
* SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS
* SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2018 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name   : r_mtr_driver_access.h
* Description : Definitions of the processing of accessing driver
***********************************************************************************************************************/
/**********************************************************************************************************************
* History : DD.MM.YYYY Version
*         : 28.09.2018 1.00
***********************************************************************************************************************/

/* Guard against multiple inclusion */
#ifndef R_MTR_DRIVER_ACCESS_H
#define R_MTR_DRIVER_ACCESS_H

/***********************************************************************************************************************
* Includes <System Includes> , "Project Includes"
***********************************************************************************************************************/
#include <stdint.h>
#include "r_mtr_parameter.h"

/***********************************************************************************************************************
* Macro definitions
***********************************************************************************************************************/
#define    MTR_DISABLE_AUTO_SET    (0)      /* Disable automatic commands update */
#define    MTR_ENABLE_AUTO_SET     (1)      /* Automatically trigger the commands update (every 1ms) */

/***********************************************************************************************************************
* Global structure
***********************************************************************************************************************/
/* Control input structure */
typedef struct
{
    uint8_t       u1_user_mode;                      /* User mode (0:ICS 1:BOARD) */
    uint8_t       u1_direction;                      /* Reference rotation direction */
    uint8_t       u1_less_switch;                    /* Soft switching */
    uint8_t       u1_flux_weakening;                 /* Flux-weakening */
    uint8_t       u1_volt_err_comp;                  /* Voltage error compensation */
    uint8_t       u1_openloop_damping;               /* Open-loop damping control */
    int16_t       s2_ref_speed_rpm;                  /* Reference speed [rpm] (mechanical) */
    uint16_t      u2_min_speed_rpm;                  /* Minimum speed [rpm] (mechanical) */
    uint16_t      u2_max_speed_rpm;                  /* Maximum speed [rpm] (mechanical) */
    uint16_t      u2_overspeed_limit_rpm;            /* Over speed limit [rpm] */
    uint16_t      u2_offset_calc_time;               /* Calculation time for current offset / 100 [us] */
    float         f4_ref_id;                         /* Reference d-axis current  [A] */
    float         f4_id_up_time;                     /* Reference d-axis current adding time[ms] */
    float         f4_limit_speed_change;             /* Change speed limit value [rad/s] */
    float         f4_freq_band_limit;                /* Motor control natural frequency limit */
    float         f4_nominal_current_rms;            /* Motor nominal current in [Arms] */
    uint16_t      u2_id_up_speed_rpm;                /* Speed threshold [rpm] to start increasing Id */
    uint16_t      u2_id_down_speed_rpm;              /* Speed threshold [rpm] to start decreasing Id */
    float         f4_id_down_time;                   /* The duration to ramp down Id to zero */
    float         f4_switch_phase_err_deg;           /* Phase error to decide sensor-less switch timing [rad] */
    float         f4_opl2less_sw_time;               /* Time to switch open-loop to sensor-less [s] */
    float         f4_ol_damping_fb_limit_rate;       /* The limit of speed feed-back rate of open-loop damping */
    mtr_parameter_t   st_motor;                      /* Motor parameters */
    mtr_design_parameter_t st_design_params;         /* Design parameter */
    mtr_ctrl_gain_t   st_ctrl_gain;                  /* Control gain */
} mtr_ctrl_input_t;

/***********************************************************************************************************************
* Global function definitions
***********************************************************************************************************************/
/***********************************************************************************************************************
* Function Name : R_MTR_InitControl
* Description   : Set motor ID and INACTIVE EVENT
* Arguments     : u1_id - Motor ID
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_InitControl(uint8_t u1_id);

/***********************************************************************************************************************
* Function Name : R_MTR_ExecEvent
* Description   : Execute event of FOC control
* Arguments     : u1_event - the event index
*                 u1_id - Motor ID
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_ExecEvent(uint8_t u1_event, uint8_t u1_id);

/***********************************************************************************************************************
* Function Name : R_MTR_ChargeCapacitor
* Description   : Wait for charging capacitor
* Arguments     : u1_id - Motor ID
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_ChargeCapacitor(uint8_t u1_id);

/***********************************************************************************************************************
* Function Name : R_MTR_SetVariables
* Description   : Set ICS input buffer to motor control structure members
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_SetVariables(void);

/***********************************************************************************************************************
* Function Name : R_MTR_SetUserifMode
* Description   : set buffer variables to User interface mode
* Arguments     : u1_user_mode -User mode (0:ICS 1:BOARD)
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_SetUserifMode(uint8_t u1_user_mode);

/***********************************************************************************************************************
* Function Name : R_MTR_SetSpeed
* Description   : Set speed reference
* Arguments     : ref_speed - speed reference [rpm]
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_SetSpeed(int16_t ref_speed);

/***********************************************************************************************************************
* Function Name : R_MTR_SetDir
* Description   : Set rotation direction
* Arguments     : dir - rotational direction
*                 u1_id - Motor ID
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_SetDir(uint8_t dir, uint8_t u1_id);

/***********************************************************************************************************************
* Function Name : R_MTR_GetSpeed
* Description   : Get speed value
* Arguments     : u1_id - Motor ID
* Return Value  : Speed[rpm]
***********************************************************************************************************************/
int16_t R_MTR_GetSpeed(uint8_t u1_id);

/***********************************************************************************************************************
* Function Name : R_MTR_GetDir
* Description   : Get rotation direction
* Arguments     : u1_id - Motor ID
* Return Value  : Rotational direction
***********************************************************************************************************************/
uint8_t R_MTR_GetDir(uint8_t u1_id);

/***********************************************************************************************************************
* Function Name : R_MTR_GetStatus
* Description   : Get status of motor control system
* Arguments     : u1_id - Motor ID
* Return Value  : Motor status
***********************************************************************************************************************/
uint8_t R_MTR_GetStatus(uint8_t u1_id);

/***********************************************************************************************************************
* Function Name : R_MTR_InputBuffParamReset
* Description   : Initialize variables when motor control reset
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_InputBuffParamReset(void);

/***********************************************************************************************************************
* Function Name : R_MTR_CtrlInput
* Description   : Store com_* variables to ICS input buffer
* Arguments     : st_ctrl_input - The pointer to the control input structure
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_CtrlInput(mtr_ctrl_input_t *st_ctrl_input);

/***********************************************************************************************************************
* Function Name : R_MTR_CtrlGainCalc
* Description   : motor control parameter & gain calculation
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_CtrlGainCalc(void);

/***********************************************************************************************************************
* Function Name : R_MTR_UpdatePolling
* Description   : Polling u1_trig_enable_write to update configurations and commands
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_UpdatePolling(void);

/***********************************************************************************************************************
* Function Name : R_MTR_AutoSetVariables
* Description   : Auto set control input buffer to motor control structure members
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_AutoSetVariables(void);
#endif /* R_MTR_DRIVER_ACCESS_H */
